import { browser, by, element, ElementFinder, ElementArrayFinder, promise, protractor, Key } from "protractor";

/**
 * Clase base con operaciones para poder acceder via protractor a los elementos del DOM. Todas las clase PO deben extender esta clase
 */
export abstract class Base {
  public static environment: any;
  public static path: any;

  /***Funciones de alto nivel *******/

  /** Retornará el elemento (ElementFinder) que coincida con el id especificado por parámetro.
   *  @param id identificador del elemento
   */
  getElementById(id: string): ElementFinder {
    return element(by.id(id));
  }

  /** Retornará el elemento (ElementFinder) que coincida con el tagName especificado por parámetro.
   *  @param tagName nombre del tag que representa al elemento a recuperar
   */
  getElementByTagName(tagName: string): ElementFinder {
    return element(by.tagName(tagName));
  }

  /** Retornará el elemento (ElementFinder) que corresponda en función de lo que diga su HTML embebido.
   *  En estos momentos solo retorna un el input asociado a un Calendar si el elemento asociado al id  especificado es un p-calendar
   *  o el input directamente asociado al id especificado.
   *  @param html código html que está dentro del elemento y nos dirá que es
   *  @param id identificador del elemento
   */
  public getElementByHtml(html: string, id: string): ElementFinder {
    if (html.indexOf("input") != -1) {
      //es un p-calendar
      this.log(`${id} es un p-calendar`);
      return this.getInputCalendar(id);
    } else {
      this.log(`${id} es un input text`);
      return this.getElementById(id);
    }
  }

  /*** Funciones para acceder elementos de formularios *****/

  /**
   * Recupera el elemento DropDown que tenga el id especificado por parámetro
   * @param id identificador del dropDown
   */
  getDropDown(id): ElementFinder {
    return this.getElementById(id)
      .all(by.css("div"))
      .first();
    // otras via pero me gustan menos return element.all(by.tagName("p-dropdown div")).first(); //element(by.id(id)).element(by.css('div.ui-dropdown'))
  }

  /**
   * Recupera la lista de opciones de un dropDown. Básicamente recupera la lista de <li>
   * Es necesario que quien invoque a este método previamente haga click() sobre el dropDown
   * @param dropDown elemento dropDown de tipo ElementFinder
   */
  getDropDownOptions(dropDown: ElementFinder): ElementArrayFinder {
    return dropDown.all(by.tagName("li")); //otra via pero me gusta menos by.css(".ui-dropdown-item")
  }

  /**
   * Recupera el valor seleccionado en el dropDown
   * @param id identificador del dropDown
   */
  getDropDownValue(id): promise.Promise<string> {
    return this.getDropDown(id)
      .element(by.tagName("label"))
      .getText();
  }

  /** Retornará el botón que contenga el texto indicado por parámetro
   *  @param textButton texto del botón
   */
  getButton(textButton: string): ElementFinder {
    return element(by.partialButtonText(textButton));
  }

  /** Retornará el botón dentro de una ventana modal y que contenga el texto indicado por parámetro
   *  @param textButton texto del botón
   */
  getDialogButton(tagName: string, textButton: string): ElementFinder {
    return element(by.cssContainingText(tagName + " button", textButton));
  }

  /** Retornará todos los botones que contengan el texto indicado por parámetro
   *  @param textButton texto del botón
   */
  getAllButtons(textButton: string): ElementArrayFinder {
    return element.all(by.partialButtonText(textButton));
  }

  /** Retornará el botón que contenga el texto Cerca*/
  getSearchButton(): ElementFinder {
    return this.getButton("Cerca"); // element(by.partialButtonText("Cerca"));
  }

  /** Retornará el botón que contenga el texto Desa*/
  getSaveButton(): ElementFinder {
    return this.getButton("Desa");
  }

  /** Retornará el botón que contenga el texto Elimina*/
  getDeleteButton(): ElementFinder {
    return this.getButton("Elimina");
  }

  /** Retornará el botón que esté visible en caso de que haya más de uno */
  getVisibleButton(textButton: string): promise.Promise<ElementFinder> {
    let elemento = null;
    return this.getAllButtons(textButton)
      .each(element => {
        element.isDisplayed().then(displayed => {
          if (displayed) {
            elemento = element;
          }
        });
      })
      .then(() => {
        return new promise.Promise((resolve, reject) => resolve(elemento));
      });
  }

  /** Recupera el inputText que tenga asociado el campo p-calendar con id especificado.
   *  @param id identificador del elemento
   */
  getInputCalendar(id: string): ElementFinder {
    return this.getElementById(id).element(by.tagName("input"));
  }

  /** Retorna el input asociado al id especificado
   *  @param id identificador del elemento
   */
  getInputText(id: string): ElementFinder {
    return this.getElementById(id);
  }

  /** Recupera el elemento InputSwitch que tenga el id especificado por parámetro
   *  @param id identificador del dropDown
   */
  getInputSwitch(id): ElementFinder {
    return this.getElementById(id)
      .all(by.css("div"))
      .first();
  }

  /** Recupera el elemento Editor que tenga el id especificado por parámetro
   *  @param id identificador del editor
   */
  getBitEditor(id): ElementFinder {
    return this.getElementById(id)
      .all(by.className("ql-editor"))
      .first();
  }

  getCheckBox(id): ElementFinder {
    return this.getElementById(id)
      .all(by.tagName("div.ui-chkbox-box"))
      .first();
  }

  getSpanByText(text: string): ElementFinder {
    return element(by.cssContainingText("span", text));
  }

  getDivByText(text: string): ElementFinder {
    return element(by.cssContainingText("div", text));
  }

  /** Recupera todos los acordeones de la página*/
  getAllAccordionTabs(): any {
    return element.all(by.css("p-accordiontab div:first-child"));
  }

  /** Recupera todos los acordeones de la página que cuelgan de un tag concreto
   *  @param tagName nombre del tag
   */
  getAllAcordionTabsFromTagName(tagName: string): ElementArrayFinder {
    return this.getElementByTagName(tagName).all(by.css("p-accordiontab div:first-child"));
  }

  /** Recupera el diágolo que cuelga de un tag concreto
   * @param tagName nombre del tag
   */
  getDialogFromTagName(tagName: string): ElementFinder {
    return this.getElementByTagName(tagName).element(by.tagName("p-dialog div:first-child"));
  }

  /** Recupera el texto que tenga el elemento HTML asociado.
   *  @param id identificador del elemento
   */
  getTextFromInput(id: string): promise.Promise<string> {
    return this.getElementById(id).getText();
  }

  /** Retorna el contenido del atributo value que tenga el elemento p-calendar con id especificado
   *  @param id
   */
  getAttributeDateFromCalendar(id: string): promise.Promise<string> {
    return this.getElementById(id)
      .element(by.tagName("input"))
      .getAttribute("value");
  }

  getCheckBoxValue(id: string): promise.Promise<string> {
    return this.getElementById(id)
      .element(by.css("span.ui-chkbox-icon"))
      .getAttribute("class")
      .then(c => {
        return new promise.Promise((resolve, reject) => resolve(c.indexOf("pi-check") == -1 ? "false" : "true"));
      });
  }

  /** Retorna el contenido del atributo value que tenga el elemento HTML con id especificado
   *  @param id identificador del elemento
   */
  getAttributeValue(id: string): promise.Promise<string> {
    return this.getElementById(id).getAttribute("value");
  }

  /** Retorna el valor del editor de HTML con id especificado
   *  @param id
   */
  getBitEditorValue(id: string): promise.Promise<string> {
    return this.getBitEditor(id).getAttribute("innerHTML");
  }

  /** Retorna la lista de opciones  del elemento p-dropdown que contengan el texto especificado por parámetro
   *  @param dropDown elemento p-dropdown del que queremos recueperar las opciones
   *  @param option texto del desplegable que nos interesa
   */
  getSelectOptionsByText(dropDown: ElementFinder, option: string): ElementArrayFinder {
    dropDown.click(); //necesario para poder mostrar todos los li>span
    let allOptions = dropDown.all(by.cssContainingText("span", option));
    return allOptions.filter(elem => {
      return elem.getText().then(text => {
        return text === option;
      });
    });
  }

  /**
   * Retorna la lista de opciones para un elemento select html que contengan el texto especificado por parámetro
   * @param select elemento select html del que queremos recuperar las opciones
   * @param text texto del desplegable que nos interesa
   */
  getSelectHTMLOptionsByText(select: ElementFinder, text: string): ElementArrayFinder {
    let allOptions = select.all(by.cssContainingText("option", text));
    return allOptions.filter(elem => {
      return elem.getText().then(textElem => {
        return textElem === text;
      });
    });
  }

  /**
   * Retorna una promesa con un array de string conteniendo el texto de los inputs en readonly especificados
   * @param inputs lista de inputs en readonly de los que extraer el texto
   */
  getFormReadOnlyValues(inputs: string[]): promise.Promise<string[]> {
    return protractor.promise.all(inputs.map(input => this.getTextFromInput(input))); //.then((values) => { return values });
  }

  /**
   * Retorna una promesa con un array de string conteniendo los valores de los inputs especificados
   * @param inputs lista de inputs de los que extraer el contenido
   */
  getFormValues(inputs: string[]): promise.Promise<string[]> {
    return protractor.promise.all(inputs.map(input => this.getInputValue(input)));
  }

  /**
   * Setea un formulario dando a cada input del array inputs el valor del array values que esté en la misma posición
   * inputs[i].setValue = values[i]
   * @param inputs lista de inputs a los que pasar valor
   * @param values lista de valores a especificar
   */
  setFormValues(inputs: string[], values: string[]): promise.Promise<void[]> {
    let i: number = 0;
    let promises = inputs.map(input => this.setInputValue(input, values[i++], true));
    return protractor.promise.all(promises).then(values => {
      return values;
    });
  }

  setFormNewValues(inputs: string[], values: string[]): promise.Promise<void[]> {
    let i: number = 0;
    let promises = inputs.map(input => this.setInputValue(input, values[i++], false));
    return protractor.promise.all(promises).then(values => {
      return values;
    });
  }

  /**
   * Recupera el valor del elemento con idenficador especificado por parámetro
   * @param input identificador del elemento del que queremos el valor
   */
  public getInputValue(input: string): promise.Promise<string> {
    return this.getElementById(input)
      .getAttribute("innerHTML")
      .then(html => {
        //this.log(`${input} tiene el valor ${html}`); para emergencias
        if (html.indexOf("dropdown") != -1) {
          //es un dropdown
          this.log(`${input} es un p-dropdown`);
          return this.getDropDownValue(input);
        } else if (html.indexOf("editor ui-widget") != -1) {
          //es un BitEditorComponent
          this.log(`${input} es un BitEditorComponent`);
          return this.getBitEditorValue(input);
        } else if (html.indexOf("ui-chkbox") != -1) {
          //es un checkbox
          this.log(`${input} es un checkbox`);
          let valor = this.getCheckBoxValue(input);
          this.log(`${input} es un checkbox con valor =>` + valor);
          return valor;
        } else if (html.indexOf("ui-inputswitch") != -1) {
          this.log(`${input} es un p-switch`);
          // si existe ui-inputswitch-checked = true, si no = false
          return html.indexOf("ui-inputswitch-checked") != -1 ? "on" : "off";
        } else if (html.indexOf("input") != -1) {
          //es un p-calendar
          this.log(`${input} es un p-calendar`);
          return this.getAttributeDateFromCalendar(input);
        } else if (html) {
          this.log(`${input} no es ningun input`);
          return html;
        } else {
          this.log(`${input} es un input text`);
          return this.getAttributeValue(input);
        }
      });
  }

  /**
   * Setea el valor especificado en el elemento con identificador indicado por parámetro. De momento
   * la función está soportada para dropdowns e input text
   * @param input identificador del elemento del que queremos setear un valor
   * @param value valor a setear en el elemento
   */
  public setInputValue(input: string, value: string, clear: boolean): promise.Promise<void> {
    return this.getElementById(input)
      .getAttribute("innerHTML")
      .then(html => {
        if (html.indexOf("dropdown") != -1) {
          //es un dropdown
          this.log(`${input} es un dropdown y le seteamos ${value}`);
          let dd = this.getDropDown(input);
          return this.getSelectOptionsByText(dd, value)
            .first()
            .click();
        } else if (html.indexOf("editor ui-widget") != -1) {
          //es un BitEditorComponent
          this.log(`${input} es un BitEditor y le seteamos ${value}`);
          let editorElement = this.getBitEditor(input);
          this.setBitEditorValue(editorElement, value);
        } else if (html.indexOf("ui-chkbox") != -1) {
          //es un checkbox
          this.getCheckBoxValue(input).then(checked => {
            this.log(`${input} es un checkbox con checked=>${checked}`);
            if (checked != value) {
              this.getCheckBox(input).click();
            }
          });
        } else if (html.indexOf("ui-inputswitch") != -1) {
          // es un p-switch
          this.getInputValue(input).then(isOn => {
            this.log(`${input} es un switch con activado=>${isOn}`);
            if (isOn != value) {
              this.getInputSwitch(input).click();
            }
          });
        } else {
          let element = this.getElementByHtml(html, input);
          if (value != null && value.length > 0) {
            // si el valor tiene algo
            this.log(`${input} le seteamos ${value}`);
            if (clear) {
              this.log(`${input} y borramos el campo`);
              // el .clear() de selenium ha dejado de funcionar en ciertos campos,
              // hay que hacerlo más artesano (Seleccionar todo y borrar)
              //element.clear();
              element.sendKeys(Key.chord(Key.CONTROL, "a", Key.DELETE));
            }
            return element.sendKeys(value);
          } else {
            //si no simplemente borramos el campo
            if (clear) {
              this.log(`${input} solo borramos el campo sin setear una mierda`);
              return element.clear();
            }
          }
        }
      });
  }

  /**
   * Setea el valor especificado en el elemento BitEditor pasado por parámetro.
   * @param element Elemento localizado que se correponde con un BitEditor
   * @param value valor a setear
   */
  public setBitEditorValue(element: ElementFinder, value: string): promise.Promise<void> {
    function setInnerHTML(arg1, arg2) {
      console.log("setInnerHTML => " + arg2);
      // if (arg1 != null) {
      arg1.innerHTML = arg2;
      // }
    }
    return browser.executeScript(setInnerHTML, element, value);
  }

  /*
  * Pulsa un botón. ESe comprueba primero que el botón es visible.
  */
  clickVisibleButton(textButton: string): promise.Promise<ElementFinder> {
    let button = this.getVisibleButton(textButton);
    button.then(r => r.click());
    return button;
  }

  /**
   * Pulsa la pestaña con el titulo pasado por parámetro.
   */
  public clickTabByTitle(title: string) {
    element
      .all(by.css(".ui-tabview-title"))
      .filter(function(elem, index) {
        return elem.getText().then(function(text) {
          return text === title;
        });
      })
      .first()
      .click();
  }

  /**
   * Pulsa la pestaña con el índice (posición) pasado por parámetro.
   */
  public clickTabByIndex(indice: number) {
    element
      .all(by.css(".ui-tabview-title"))
      .filter(function(elem, index) {
        return index === indice;
      })
      .first()
      .click();
  }

  /**
   * Selecciona una fecha dentro de un popup de un date picker. Es necesario cuando no se puede hacer un setter directamente
   * del campo porque esta deshabilido y solo se puede cambiar clickando en el calendario.
   */
  public chooseDatePickerDate(fieldName: string, day: number, month: number, year: number) {
    this.log(`chooseDatePickerDate of field ${fieldName} to ${day}-${month}-${year}`);
    this.getInputText(fieldName).click();

    // element.all(by.cssContainingText("select.ui-datepicker-month option", "marzo")).first().click();
    element
      .all(by.css("select.ui-datepicker-month option"))
      .get(month)
      .click();
    element
      .all(by.cssContainingText("select.ui-datepicker-year option", "" + year))
      .first()
      .click();
    //element.all(by.css("table.ui-datepicker-calendar td.ng-star-inserted:not(.ui-datepicker-other-month)")).get(day-1).click();

    element
      .all(by.css("table.ui-datepicker-calendar td:not(.ui-state-disabled) a"))
      .get(day - 1)
      .click();
    // Lo siguinte falla si , por ejemplo, el dia es 1 y se encuentra con anterioridad el dia 31 ya que 31 contiene el numero 1
    // element.all(by.cssContainingText("table.ui-datepicker-calendar a", ''+day)).first().click();
  }

  /**
   * Sube un fichero con el nombre y indice (en caso de múltiples ficheros) pasado por parámetro
   * El fichero se encuentra en la carpeta raiz de los tests (e2e).
   */
  public uploadFile(filename: string = "protractor.png", index: number = 0) {
    var path = require("path");
    var absolutePath = path.resolve(Base.path, filename);

    var fileElem = element.all(by.css('input[type="file"]')).get(index);
    // Unhide file input
    browser.executeScript(
      "arguments[0].style.visibility = 'visible'; arguments[0].style.height = '1px'; arguments[0].style.width = '1px';  arguments[0].style.opacity = 1",
      fileElem.getWebElement()
    );
    fileElem.sendKeys(absolutePath);

    // take a breath
    browser.driver.sleep(100);

    this.getButton("Carrega").click();
  }

  /*** Funciones para acceder a listas de cosas *****/

  /** Retornará las filas visibles (no hidden) de la tabla en la página. Solo usar cuando en la página hay una sola tabla. Típico de filtro + listado*/
  getTableRows(): ElementArrayFinder {
    return element.all(by.tagName("table tbody tr")).filter(element => {
      return element.isDisplayed().then(v => {
        return v;
      });
    });
  }

  /** Retornará las filas visibles (no hidden) de una tabla que forme parte del tag especificado por parámetro
   *  @param tagName nombre del tag que contiene la tabla que nos interesa
   */
  getTableRowsByTagName(tagName: string): ElementArrayFinder {
    return element(by.tagName(tagName))
      .all(by.tagName("table tbody tr:not(.ui-datatable-emptymessage-row)"))
      .filter(element => {
        return element.isDisplayed().then(v => {
          return v;
        });
      });
  }

  /** Retornará las filas visibles (no hidden) de una tabla que forme parte del tag especificado por parámetro y que además tengan aplicada la clase específica
   *  @param tagName nombre del tag que contiene la tabla que nos interesa
   *  @param selector selector css a aplicar
   */
  getTableRowsByTagNameAndSelector(tagName: string, selector: string): ElementArrayFinder {
    return element(by.tagName(tagName))
      .all(by.css(selector))
      .filter(element => {
        return element.isDisplayed().then(v => {
          return v;
        });
      });
  }

  /** Retornará el indice de la fila dentro de la tabla con el tag especificado en el que coincida el valor de la columna con el indice y valor pasado por parámetro
   *  @param tagName nombre del tag que contiene la tabla que nos interesa
   *  @param index indice de la columna en la que se busca el valor
   *  @param value valor de la celda cuya fila se quiere buscar
   */
  getTableRowIndexByCellIndexValue(tagName: string, index: number, value: string): promise.Promise<number> {
    let indice = -1;
    return element(by.tagName(tagName))
      .all(by.tagName("table tbody tr"))
      .each((element, indexRow) => {
        element.all(by.css("td:nth-child(" + (index + 1) + ") span")).each((elementCell, indexCell) => {
          elementCell.getAttribute("innerHTML").then(html => {
            if (html == value && indice == -1) indice = indexRow;
          });
        });
      })
      .then(() => {
        return new promise.Promise((resolve, reject) => resolve(indice));
      });
  }

  /**
   * Elimina la fila en la posicion especificad y de la tabla con el tag especificado.
   * Utilizado en los listados donde la primera columna es un checkbox para seleccionar la fila a eliminar.
   * @param tagName nombre del tag que contiene la tabla que nos interesa
   * @param index indice de la fila a eliminar
   */
  public deleteTableRowByIndex(tagName: string, index: promise.Promise<number>) {
    this.getTableRowsByTagName(tagName)
      .get(index)
      .all(by.css("div.ui-chkbox-box"))
      .click();
    // NOTA es posible que algunos formularios tengan mas de un boton Elimina y haga falta indicar que boton Elimina se pulsa
    this.getAllButtons("Elimina")
      .last()
      .click();
    this.acceptAlert();
  }

  /**
   * Pulsa la fila de una tabla localizándola segun el indice y valor de columna pasados por parámetros.
   * @param tagName nombre del tag que contiene la tabla que nos interesa
   * @param index indice de la columna a buscar para saber la posición de la fila a eliminar
   * @param value valor de la columna a buscar para saber la posición de la fila a eliminar
   */
  public clickTableRowByCellIndexValue(tagName: string, index: number, value: string): promise.Promise<number> {
    let indice = this.getTableRowIndexByCellIndexValue(tagName, index, value);
    expect(indice).toBeGreaterThan(-1);
    this.getTableRowsByTagName(tagName)
      .get(indice)
      .click();
    return indice;
  }

  /** Retornará una promesa con el número de elementos que estén visibles en la lista especificada
   *  @param list lista de elementos a analizar
   */
  getCountVisibleElements(list: ElementArrayFinder): promise.Promise<number> {
    return list
      .filter(element => {
        return element.isDisplayed().then(v => {
          return v;
        });
      })
      .count();

    /* forma artesanal a modo de aprender :-)
        let countVisibles: number = 0;
        return list
            .each(element => {                
                element.isDisplayed().then(v => {if (v) countVisibles++;}); //vamos anotando los visibles
            })
            .then(() => {
                this.log(`List has ${countVisibles} visible elements`);
                return new promise.Promise((resolve, reject) => resolve(countVisibles)); //retornamos una promesa con los visibles
            });         
            */
  }

  /**
   * Retorna una promesa con true si todos los elementos de la lsita contienen el texto esperado y false en caso contrario
   * @param list ElementArrayFinder con la lista de elementos del dom a analizar
   * @param text texto que deberían contener todos los elementos del dom especificados
   */
  allElementsContainingText(list: ElementArrayFinder, text: string): promise.Promise<boolean> {
    let contain: boolean = true; //asumimos que se cumple
    let count: number = 0;
    return list
      .each(element => {
        count++;
        element.getText().then(elementText => {
          if (!elementText.includes(text)) {
            this.log(`texto que no cumple ${elementText}`);
            contain = false;
          }
        });
      })
      .then(() => {
        this.log(`elementos analizados totales ${count}`);
        return new promise.Promise((resolve, reject) => resolve(contain)); //retornamos una promesa con el resultado
      });
  }

  /*** Funciones varias *****/

  expectIsPresent(element: ElementFinder, error: string = null) {
    element.isPresent().then(e => {
      expect(e).toBeTruthy((error ? error : "element") + " should be present");
    });
  }

  expectIsNotPresent(element: ElementFinder, error: string = null) {
    element.isPresent().then(e => {
      expect(e).toBeFalsy((error ? error : "element") + " should not be present");
    });
  }

  expectVisibleButtonIsPresent(textButton: string) {
    this.getVisibleButton(textButton).then(r => {
      expect(r).toBeDefined("Element " + textButton + " should be present");
    });
  }

  expectTabIsActive(tabs: string[]) {
    for (var tab of tabs) {
      expect(element(by.xpath("//li[contains(@class, 'ui-state-active')][.//span[contains(text(), '" + tab + "')]]")).isPresent()).toBeTruthy();
    }
  }

  expectTabIsEnabled(tabs: string[]) {
    for (var tab of tabs) {
      expect(element(by.xpath("//li[contains(@class, 'ui-state-default')][.//span[contains(text(), '" + tab + "')]]")).isPresent()).toBeTruthy();
    }
  }

  expectTabIsDisabled(tabs: string[]) {
    for (var tab of tabs) {
      expect(element(by.xpath("//li[contains(@class, 'ui-state-disabled')][.//span[contains(text(), '" + tab + "')]]")).isPresent()).toBeTruthy();
    }
  }

  /** Retorna true si el DIV de error es visible */
  expectExisteError(errorMessage: string[] = []): void {
    //let divError: ElementFinder = element(by.cssContainingText("div.ui-messages-error span.ui-messages-detail", "Error:"));
    if (errorMessage.length > 0) {
      for (let error of errorMessage) {
        let divError: ElementFinder = element(by.cssContainingText("div.ui-messages-error span.ui-messages-detail", error));
        expect(divError.isPresent()).toBeTruthy();
      }
    } else {
      let divError: ElementFinder = element(by.css("div.ui-messages-error span.ui-messages-detail"));
      expect(divError.isPresent());
    }
  }

  waitForAngular() {
    return browser.waitForAngular();
  }

  /** Navegará a la url que indiquemos por parámetro
   *  @param url URL a la que navegar incluido el contextPath pero sin el host y puerto
   */
  navigateTo(url: string): promise.Promise<any> {
    return browser.get(url);
  }

  reload(): any {
    return browser.refresh();
  }

  scrollDown(x, y): any {
    browser.executeScript(`window.scrollTo(${x},${y});`);
  }

  scrollBottom(): any {
    browser.executeScript("window.scrollTo(0,document.body.scrollHeight);");
  }

  maximize(): any {
    browser.driver
      .manage()
      .window()
      .maximize();
  }

  getCurrentUrl(): promise.Promise<string> {
    return browser.getCurrentUrl().then(url => {
      this.log(`navegando hacia ${url}`);
      return url;
    });
  }

  /**
   * Retornará el ID de un objeto y que se encuentra al final de la URL actual.
   */
  getIdFromCurrentUrl(): promise.Promise<number> {
    return browser.getCurrentUrl().then(url => {
      this.log(`hemos obtenido la url: ${url}`);
      let indiceUltimaBarra: number = url.lastIndexOf("/");
      return +url.substring(indiceUltimaBarra + 1);
    });
  }

  acceptAlert(texto: string = null): promise.Promise<void> {
    var EC = protractor.ExpectedConditions;
    browser.wait(EC.alertIsPresent(), 4000, "Alert no presente :(");
    return browser
      .switchTo()
      .alert()
      .accept();
    /*var alertDialog = browser.switchTo().alert();
        if (texto) {
            return alertDialog.getText().then(textoAlert => {
                if (textoAlert.indexOf(texto) == -1) {
                    fail("expected '" + textoAlert + "' to contain '" + texto + "'");
                } else {
                    this.log("alert text matches");
                    return alertDialog.accept();
                }
            });
            
        } else {
            return alertDialog.accept();
        }*/
  }

  /**
   * Retornará true si el elemento pasado por parámetro tiene el atributo "disabled" a true.
   * Util para comprobar si un botón está activo o no.
   */
  isDisabled(element: ElementFinder): promise.Promise<boolean> {
    return element.getAttribute("disabled").then(attribute => {
      return attribute == "true";
    });
  }

  /**
   * Sitúa el cursor sobre el elemento.
   */
  mouseOver(element: ElementFinder) {
    protractor.browser
      .actions()
      .mouseMove(element)
      .perform();
  }

  /**
   * Sitúa el cursor sobre el elemento.
   */
  mouseClick(element: ElementFinder, offset = { x: 0, y: 0 }) {
    protractor.browser
      .actions()
      .mouseMove(element, offset)
      .mouseDown()
      .mouseUp()
      .perform();
  }

  /**
   * Sitúa el cursor sobre el elemento.
   */
  mouseCtrlClick(element: ElementFinder, offset = { x: 0, y: 0 }) {
    protractor.browser
      .actions()
      .mouseMove(element, offset)
      .keyDown(protractor.Key.CONTROL)
      .mouseDown()
      .mouseUp()
      .perform();
  }

  printOptionsCount(optionsCount: promise.Promise<number>) {
    optionsCount.then(count => {
      this.log(`el dropdown tiene ${count} opciones `);
    });
  }

  printHowManyRows(rowsCount: promise.Promise<number>) {
    rowsCount.then(count => {
      this.log(`la tabla tiene ${count} filas`);
    });
  }

  log(texto: string) {
    Base.environment && Base.environment.debug && console.log(texto);
  }
}
