import { EventEmitter, Output, Input, HostListener, AfterContentInit } from "@angular/core";
import { FormGroup } from "@angular/forms";
import { Router, ActivatedRoute } from "@angular/router";
import { Observable, from } from "rxjs";

import { BaseComponent } from "./base.component";
import { FormType, PageStatus } from "../../shared/global.constants";
import { BitToolbarComponent } from "../../shared/component/bit-toolbar.component";
import { BitMessage } from "../../shared/data/bitmessage";
import { Item } from "../../shared/data/item";
import { Error } from "../../shared/data/error";
import { CustomValidators } from "../../shared/service/custom-validators.service";
import { GeneralUtils } from "../../shared/service/general-utils.service";
import { MessageService } from "../../shared/service/message.service";
import { ToolbarService } from "../../shared/service/toolbar.service";
import { BaseCRUDServiceInterface } from "../shared/base-interface.service";

/**
 * Clase base para implementar formularios de elementos. Provee el 95% de la funcionalidad necesaria.
 */
export abstract class BaseFormComponent extends BaseComponent implements AfterContentInit {
  myForm: FormGroup;
  object: Item;
  breadcrumb: any[] = []; // contiene la miga de pan de nuestro componente
  //heredado! status: PageStatus;     // determina en qué estado está la página: PageStatus.Init, PageStatus.SEARCHING, PageStatus.FINISH_SEARCH, PageStatus.ERROR
  messages: Map<PageStatus, string>; // mensajes a indicar en el footer

  protected formType: FormType;
  protected dataService: BaseCRUDServiceInterface;
  protected paramsDataService: any;
  protected router: Router;
  protected activatedRoute: ActivatedRoute;
  protected messageService: MessageService;
  @Output()
  onSave: EventEmitter<any> = new EventEmitter<any>(); // envía el objeto actualizado cada vez que se inserta/actualiza
  @Input()
  dialogMode: boolean; // indica si el formulario se ha abierto en un diálogo emergente
  @Input()
  reset: EventEmitter<any>; // recibe eventos para resetear el componente

  protected myTabs: any = {}; // pestañas definidas en el componente (ver registerTab() e initTabs())
  protected initialTab: string; // pestaña cargada inicialmente (indicada por parámetro)
  protected loadedTab: any = {}; // pestañas con datos cargados (todas se cargan de forma perezosa)
  currentTab: number; // pestaña mostrada actualmente

  /**
   * Constructor de la Clase.
   * @param dataService: Servicio que implementa la interface BaseCRUDServiceInterface.
   * @param router: Objeto de tipo Router para poder navegar (p. ej., al listado).
   * @param activateRoute: Objeto de tipo ActivatedRoute que permite acceder a los parámetros de la URL con los que
   *        hemos accedido a este componente o para navegar.
   * @param notificationService: Objeto de tipo NotificationService para poder mostrar diálogos al usuario.
   */
  constructor(
    dataService: BaseCRUDServiceInterface,
    router: Router,
    activatedRoute: ActivatedRoute,
    messageService: MessageService,
    toolbarService?: ToolbarService,
    componentId?: string
  ) {
    super(messageService, toolbarService, componentId);
    this.dataService = dataService;
    this.router = router;
    this.activatedRoute = activatedRoute;
    this.toolbar = {
      //mínimos toolbar de un formulario
      save: { visible: true, enable: true },
      delete: { visible: false, enable: true },
      print: { visible: true, enable: true }
    };
    // mensajes para el footer
    this.messages = new Map<PageStatus, string>();
    this.messages.set(PageStatus.Init, "Premi el botó Desa per actualitzar les dades");
    this.messages.set(PageStatus.Searching, "Enviant la sol·licitud...");
    this.messages.set(PageStatus.FinishSearch, "Dades actualitzades satisfactòriament");
    this.messages.set(PageStatus.ReadOnly, "Formulari no editable");
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  /** Retrocede a la pantalla anterior (normalmente, el listado de elementos de la misma clase). */
  abstract goBack(): void;

  ngAfterContentInit() {
    console.log("ngAfterContentInit base-form.component");
    this.reset &&
      this.reset.subscribe(x => {
        this.object = null;
        this.myForm && this.myForm.markAsPristine(); // el formulario vuelve a estar sincronizado con el servidor
        this.myForm && this.myForm.markAsUntouched(); // idem
        this.init();
      });
    this.initTabs();
  }

  /**
   * Inicializa el formulario.
   * 1. Identifica si el formulario se carga para insertar un nuevo elemento o consultar/actualizar uno existente.
   * 2. En este segundo caso, lee el elemento con el identificador indicado por parámetro desde el servidor.
   * 3. Una vez cargado el elemento, invoca el método postInit con el resultado devuelto por el servidor.
   * @param id: Identificador del elemento. En el caso de una inserción, puede ser cero o undefined.
   */
  init(id?: number): void {
    console.log("init base-form.component");
    this.formType = FormType.Insert;
    this.status = PageStatus.Init;
    if (id) {
      // algunos proyectos utilizan identificadores negativos para datos del sistema!
      this.dataService
        .get(id, this.paramsDataService)
        .subscribe(result => this.postInit(result), error => this.messages.set(PageStatus.Error, error));
    } else {
      this.postInit();
    }
  }

  /** Cuando se pulsa un botón del toolbar. */
  onToolbarButtonPressed(button: string) {
    console.log("Button pressed:" + button);
    if (button === BitToolbarComponent.SAVE_BUTTON) {
      this.save();
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      this.delete();
    } else if (button === BitToolbarComponent.PRINT_BUTTON) {
      this.handlePrint();
    } else if (button === BitToolbarComponent.HELP_BUTTON) {
      this.help();
    } else {
      this.otherAction(button);
    }
  }

  /**
   * Inserta o actualiza el registro en el servidor.
   * Una vez insertado o actualizado, se invoca al método postSave con el resultado de la operación.
   */
  save(): void {
    if (!this.myForm.valid) {
      let errores = CustomValidators.validateForm(this.myForm);
      this.messageService.sendError(
        new Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores)
      );
    } else {
      if (this.preSave()) {
        //console.log('save', this.object);
        this.dataService.save(this.object, this.formType, this.paramsDataService).subscribe(
          result => this.postSave(result),
          error => {
            this.status = PageStatus.Error;
          }
        );
      }
    }
  }

  /** Elimina el registro en el servidor. Una vez eliminado, se invoca al método postDelete. */
  delete(): void {
    let p = this.messageService.confirm(
      "Està segur que vol eliminar aquest registre? Aquesta acciò no es podrà desfer?"
    );
    let local = this;
    p.then(function(confirmacion) {
      if (confirmacion) {
        local.dataService.delete(local.object.id, local.paramsDataService).subscribe(
          result => {
            local.postDelete();
          },
          error => {
            local.status = PageStatus.Error;
          }
        );
      }
    });
  }

  /** Muestra la ayuda asociada a la pantalla. */
  help(): void {}

  handlePrint() {
    window.print();
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {
    console.warn(`otherAction ${action} pressed`);
  }

  /** Limpia el formulario, estableciendo los valores como si hubieramos entrado por primera vez. */
  cleanForm(): void {
    //¿¿Algo que hacer??
  }

  /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
  get isEditMode(): boolean {
    return this.formType === FormType.Update;
  }
  /**
   * Devuelve true si hay algún valor cambiado en la ficha.
   * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
   */
  get isDataChanged(): boolean {
    return this.myForm.dirty;
  }

  /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
  get isFormValid(): boolean {
    //console.log('isFormValid', this.myForm.valid, this.myForm);
    return this.myForm.valid;
  }

  /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
  canDeactivate(): Observable<boolean> | boolean {
    if (this.isDataChanged) {
      //si hay cambios, mostramos la ventana de diálogo
      let p = this.messageService.confirm(
        "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?"
      );
      let o = from(p);
      return o;
    } else {
      // si no hay cambios, no la mostramos
      return true;
    }
  }

  @HostListener("window:beforeunload")
  unloadWithUnsavedChanges($event: any) {
    let msg = "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?";
    if (this.isDataChanged) {
      ($event || window.event).returnValue = msg;
      return msg;
    }
    return undefined;
  }

  /** Método que se ejecuta después de la lectura de los datos desde el servidor. */
  protected postInit(result?: any) {
    result = result || this.object;
    this.object = result;
    this.formType = result && result.id ? FormType.Update : FormType.Insert;
    if (this.toolbar[BitToolbarComponent.DELETE_BUTTON])
      this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = this.formType == FormType.Update && !this.dialogMode; // no permitimos borrar en modo diálogo
    if (this.toolbar[BitToolbarComponent.SAVE_BUTTON]) this.toolbar[BitToolbarComponent.SAVE_BUTTON].enable = true; //this.isFormValid; //si el form es valido activamos el botón
    this.updateToolbar();
  }

  /** Método que se ejecuta antes de la inserción/modificación de los datos en el servidor. */
  protected preSave() {
    // por ejemplo, limpiar objetos anidados que no tengan ID (workaround para formularios)
    return true;
  }

  /** Método que se ejecuta después de la inserción/modificación de los datos en el servidor. */
  protected postSave(result: any, avoidMessage?: boolean) {
    this.object = result;
    avoidMessage ||
      this.messageService.sendNotification(new BitMessage("Informació", "Les dades s'han desat correctament."));

    // tras un guardado, marcamos el formulario como virgen para que no nos avise si intentamos abandonar la pantalla
    this.myForm.markAsPristine(); // el formulario vuelve a estar sincronizado con el servidor
    this.myForm.markAsUntouched(); // idem
    if (this.formType == FormType.Insert) {
      // navegamos a la ficha en modo edición. Esto permite que se sustituya la URL y evitamos efectos colaterales
      !this.dialogMode &&
        this.router.navigate([GeneralUtils.getPathFromURL(this.activatedRoute.snapshot.url), result.id], {
          preserveQueryParams: this.isPreserveQueryParams(),
          relativeTo: this.activatedRoute.parent
        });
      // si estamos en modo diálogo, hacemos un nuevo init con la entidad insertada
      this.dialogMode && this.init(result.id);
    }
    this.onSave && this.onSave.emit(this.object);
  }

  /** Método que se ejecuta después de la eliminación del elemento en el servidor. */
  protected postDelete() {
    this.messageService.sendNotification(new BitMessage("Informació", "Les dades s'han eliminat correctament."));
    !this.dialogMode && this.goBack();
  }

  /* Devuelve el breadcrumb solo si procede (por ejemplo, en modo diálogo no queremos breadcrumb). */
  get safeBreadcrumb() {
    return !this.dialogMode && this.breadcrumb;
  }

  // Métodos relacionados con pestañas (tabs) //

  /** Registra una pestaña de visualización de información. */
  protected registerTab(index: number, id: string) {
    this.myTabs[id] = index;
  }

  // inicialización de pestañas (si hay)
  protected initTabs() {
    this.currentTab = 0;
    this.initialTab = this.activatedRoute.snapshot.params["tab"];
    for (let tab in this.myTabs) {
      let isInitial = this.isInitialTab(tab);
      this.loadedTab[tab] = isInitial;
      if (isInitial) {
        this.currentTab = this.myTabs[tab];
      }
    }
  }

  /** Indica si la pestaña correspondiente es la inicial. */
  public isInitialTab(tab: string) {
    return this.initialTab === tab;
  }

  /** Indica si la pestaña correspondiente está cargada. */
  public isLoadedTab(tab: string) {
    return this.loadedTab[tab];
  }

  /** Controla cuando el usuario cambia de tab. */
  public changeTab(e) {
    console.log("change tab to number {}", e.index);
    this.currentTab = e.index;
    for (let tab in this.myTabs) {
      this.loadedTab[tab] = this.loadedTab[tab] || e.index == this.myTabs[tab];
    }
  }

  protected isPreserveQueryParams() {
    return true;
  }
}
