import { Input, SimpleChange, ViewChild, EventEmitter } from "@angular/core";

import { BaseComponent } from "./base.component";
import { Item } from "../../shared/data/item";
import { MessageService } from "../../shared/service/message.service";
import { ToolbarService } from "../../shared/service/toolbar.service";
import { GeneralUtils } from "../../shared/service/general-utils.service";
import { FormType } from "../../shared/global.constants";
import { BitToolbarComponent } from "../../shared/component/bit-toolbar.component";
/**
 */
export abstract class BaseSublistComponent extends BaseComponent {
  lista: Item[]; // lista de elementos a representar
  displayInnerForm: boolean = false; // si tenemos que mostrar el form para editar el objeto en linea o no
  modeInnerForm: FormType; // modo en que mostramos el inner form
  itemSelected: Item; // objeto a enviar al InnerForm

  @ViewChild("tableview")
  tableView: any;
  @Input()
  parent: Item; // objeto padre que es dueño de la lista
  @Input()
  parentField: string; // propiedad de los elementos que establece la relación con su padre
  @Input()
  initLoad: boolean; // cuando sea true la lista se cargará
  @Input()
  itemsPreloaded: Item[]; // lista precargada de elementos que el padre nos inyecta
  @Input()
  femenino: boolean = false; // si tenemos que mostrar los textos en femenino o no (p. ej., Nou vs Nova)
  @Input()
  forceReload: EventEmitter<any>; // opcional, eventos a esta cola nos dicen que debemos forzar una recarga
  @Input()
  reloadFromServer: boolean = true; //indica si una vez añadido un elemento, llamaos al servidor para recargar la lista. Por defecto sí lo hace.
  //tendremos que recargar siempre que la lista se ordene con algún criterio diferente al id.
  protected loadedParentId: number; // controla si hemos cargado la lista de elementos para no volver a cargarla
  protected selectedItems: number[]; // elementos checkeados

  constructor(
    protected generalUtils: GeneralUtils,
    protected messageService: MessageService,
    toolbarService?: ToolbarService,
    componentId?: string
  ) {
    super(messageService, toolbarService, componentId);

    this.toolbar = {};
    this.toolbar[BitToolbarComponent.NEW_BUTTON] = { enable: true, visible: true };
    this.toolbar[BitToolbarComponent.DELETE_BUTTON] = { enable: true, visible: false };
    this.updateToolbar();
  }

  /** Función para mostrar el formulario del item. Podría ser via Dialog emergente, llamando a otro route, etc.. */
  abstract gotoForm(data?: any);
  /** Función que la clase que extienda esta clase debe implementar como cargar la lista */
  abstract callLoadItems(parent: Item);
  /** Función que la clase que extienda esta clase debe implementar como eliminar los elementos seleccionados */
  abstract callDeleteItems(parent: Item, items: number[]);
  /** Función que la clase que extienda esta clase debe implementar como guardar un elemento */
  abstract callSaveItem(parent: Item, item: Item, mode: FormType);

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  /** Controlamos si la propiedad initLoad cambia a true para cargar la lista de documentos */
  ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
    // console.log('BaseSublist OnChanges', changes);
    for (let propName in changes) {
      let changedProp = changes[propName];
      if (propName === "initLoad" || propName === "parent") {
        if (this.initLoad && this.parent && this.parent.id && this.loadedParentId !== this.parent.id) {
          // nos dicen de cargar y no hemos cargado y el padre se ha cargado
          this.loadItems();
        }
      }
      if (propName === "itemsPreloaded") {
        this.lista = changedProp.currentValue;
      }
      if (propName === "forceReload") {
        this.forceReload &&
          this.forceReload.subscribe(
            function(x) {
              this.loadedParentId === this.parent.id && this.loadItems();
            }.bind(this)
          );
      }
    }
  }
  /** Nos presionan algún botón del toolbar */
  onToolbarButtonPressed(button: string) {
    console.log(`Button pressed: ${button}`);
    if (button === BitToolbarComponent.NEW_BUTTON) {
      this.gotoForm();
      //this.openForm(BitDocument.newObject(), true);
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      let p = this.messageService.confirm(
        "Està segur que vol eliminar els elements seleccionats? Aquesta acciò no es podrà desfer."
      );
      let local = this;
      p.then(function(confirmacion) {
        if (confirmacion) {
          local.deleteItems(local.selectedItems);
        }
      });
      //this.deleteItems(this.selectedItems);
    } else {
      this.otherAction(button);
    }
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {
    console.warn(`otherAction ${action} pressed`);
  }

  /** Nos checkean algún item del listado */
  onItemChecked(selectedItems: number[]) {
    // console.log('element selected');
    this.selectedItems = selectedItems;
    if (this.toolbar[BitToolbarComponent.DELETE_BUTTON]) {
      if (selectedItems !== null && selectedItems.length > 0) {
        this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = true;
      } else {
        this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
      }
      this.updateToolbar();
    }
  }
  /** nos piden cerrar el formulario */
  closeForm(doc: any) {
    this.displayInnerForm = false;
  }
  /** Cargamos la lista de documentos del usuario */

  loadItems(): void {
    this.callLoadItems(this.parent).subscribe(result => {
      this.lista = result;
      this.loadedParentId = this.parent.id;
    });
  }
  /** Elimina los documentos seleccionados */
  deleteItems(items: any[]): void {
    console.log("delete items");
    this.callDeleteItems(this.parent, items).subscribe(result => {
      this.lista = this.lista.filter(item => items.indexOf("" + item.id) < 0);
      this.postDelete(result);
    });
  }
  /** Inserta o actualiza el documento */
  saveItem(item: Item): void {
    this.preSave(item);
    //item.idUser = this.parent.id;  //mapeamos el identificador del usuario en el objeto BitDocument
    this.callSaveItem(this.parent, item, this.modeInnerForm).subscribe(result => {
      this.displayInnerForm = false;
      if (this.reloadFromServer) {
        this.loadItems();
      } else {
        if (this.modeInnerForm === FormType.Insert) {
          if (!this.lista) this.lista = []; //la lista no estará inicializada si el componente se cargó sin elementos
          this.lista.push(result);
        } else {
          let index = this.lista.findIndex(item2 => item2.id === item.id); //posicion del objeto en el array
          if (index != -1) {
            //controlamos que el elemento a insertar esté en la sublista
            this.lista.splice(index, 1, item); //sustituimos el objeto de la posición por el nuevo que nos llega del servidor
          }
        }
      }
      this.postSave(result);
    });
  }
  /** Función para hacer algo antes de guardar un item */
  preSave(item: Item) {
    return item;
  }
  /** Función para hacer algo después de haber guardado */
  postSave(result: Item) {
    console.log("postSave llamado");
  }
  /** Función para hacer algo después de haber eliminado los elementos seleccionados */
  postDelete(result: any) {
    console.log("postDelete llamado");
    this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
    this.updateToolbar();
    if (this.tableView != null) this.tableView.selectedItems = [];
  }
}
