import {ViewChild,
        Input,
        Output,
        OnDestroy,
        EventEmitter}           from '@angular/core';
import {Subscription}           from 'rxjs';
import {DataTable}              from 'primeng/components/datatable/datatable';
import * as moment              from 'moment';
import {BaseCRUDServiceInterface}   from '../shared/base-interface.service';
import {Domain}                 from '../../shared/data/domain';
import {Item}                   from '../../shared/data/item';
import {DomainCachedService}    from '../../shared/service/domain-cached.service';
import {TableViewService}       from '../../shared/service/tableview.service';        
import {BaseFilter}             from '../shared/base.filter';

/**
 * Clase base de los componentes TableView
 */
export class BaseTableViewComponent implements OnDestroy {

    @ViewChild('dataTable') public dataTable: DataTable;
    
    items: Item[];

    @Input() totalItems: number = 0;
    @Output() private onPageChange = new EventEmitter<Item>();

    protected dataService: BaseCRUDServiceInterface;
    protected domainCachedService: DomainCachedService;
    protected tableViewService: TableViewService;

    csvSeparator: string = ',';

    private subscriptionMessages: Subscription;
    protected RESULTADOS_PAGE = BaseFilter.RESULTADOS_PAGE; // se hace visible esta variable a cualquier subclase de tableview que pudiera necesitarla

    constructor(
                domainCachedService: DomainCachedService,
                tableViewService: TableViewService) {

      this.domainCachedService = domainCachedService;
      this.tableViewService = tableViewService;
      this.subscriptionMessages = tableViewService.observableExport.subscribe(
        message => {
            this.exportCSV();
        });
    }

    exportCSV() {

        let dt = this.dataTable;
        if (dt == null) {
            alert("Exportació no disponible");
            return;
        }
        let data = this.items;

        let csv = '';

        //headers
        for(let i = 0; i < dt.columns.length; i++) {
            if(dt.columns[i].field && !dt.columns[i].hidden) {
                csv += dt.columns[i].header || dt.columns[i].field;

                if(i < (dt.columns.length - 1)) {
                    csv += this.csvSeparator;
                }
            }
        }

        //body        
        dt.value.forEach((record, i) => {
            csv += '\n';
            for(let i = 0; i < dt.columns.length; i++) {
                if(dt.columns[i].field && !dt.columns[i].hidden) {

                    let cell = this.getColumna(record, dt.columns[i].field) || dt.resolveFieldData(record, dt.columns[i].field);
                    if (cell == null) {
                        cell = '';
                    }
                    cell = cell.toString();
                    if (cell.indexOf(",") != -1) {
                        cell = "\"" + cell + "\"";
                    }

                    csv += cell;
                    
                    if(i < (dt.columns.length - 1)) {
                        csv += this.csvSeparator;
                    }
                }
            }
        });

        let blob = new Blob(["\ufeff", csv], {
            type: 'text/csv;charset=utf-8;'
        });

        if(window.navigator.msSaveOrOpenBlob) {
            navigator.msSaveOrOpenBlob(blob, dt.exportFilename + '.csv');
        }
        else {
            let link = document.createElement("a");
            link.style.display = 'none';
            document.body.appendChild(link);
            if(link.download !== undefined) {
                link.setAttribute('href', URL.createObjectURL(blob));
                link.setAttribute('download', dt.exportFilename + '.csv');
                link.click();
            }
            else {
                csv = 'data:text/csv;charset=utf-8,' + csv;
                window.open(encodeURI(csv));
            }
            document.body.removeChild(link);
        }
    }

    getDataTable(): DataTable {
        return null;
    }

    getColumna(record: Item, field: String) {
        return null;
    }

    getColumnaDominio(dominio: Domain[], valor: Object) {
        let domainMatch = dominio && dominio.find(obj => {return obj.value == valor});
        if(domainMatch) {
            return domainMatch.label;
        } else if(valor) {
           return new String(`??${valor}??`);
        }
        return '';
    }

    ngOnDestroy() {
        this.subscriptionMessages.unsubscribe();
    }

    pageChanged(event): void {
        console.log("pageChanged("+event+")");
        this.onPageChange.emit(event.page+1);
    }

    /**
     * Método de ordenación de campos de fecha.
     * @see Adaptación de: https://stackoverflow.com/questions/41181277/primeng-datatable-custom-sorting-or-filtering-angular-2
     */
    dateSort(event, field) {
        const comparer = function (a, b): number {
            const formatedA = a[field] ? moment(a[field], 'DD/MM/YYYY').format('YYYY-MM-DD') : '-';
            const formatedB = b[field] ? moment(b[field], 'DD/MM/YYYY').format('YYYY-MM-DD') : '-';
            // let result = -1; if (moment(formatedB).isBefore(formatedA, 'day')) { result = 1; }
            const result = formatedA.localeCompare(formatedB);
            return result * event.order;
        };

        const strcmp = function (a, b): number {
            if (a.toString() < b.toString()) { return -1; }
            if (a.toString() > b.toString()) { return  1; }
            return 0;
        };

        this.items.sort(comparer);
        this.items = [...this.items];
    }

}
