import { PageStatus, FormType } from "../../shared/global.constants";
import { FactoryCreateNewObject, FactoryCrateToolbar } from "./types";

export interface BaseFormState<T> {
  item: T;
  toolbar: any;
  status: PageStatus;
  readOnly: boolean;
  formType: FormType;
  currentTab: number;
  loaded: boolean;
  updated: boolean;
}

export class BaseFormAdapter<T> {
  constructor(
    protected fnCreateObject: FactoryCreateNewObject<T>,
    protected fnGetToolbarObjectLoaded?: FactoryCrateToolbar<T>
  ) {}

  getInitialState<S extends BaseFormState<T>>(optional?: any): S {
    const initialState = Object.assign(
      {
        item: this.fnCreateObject(),
        toolbar: this.initToolbar(),
        status: PageStatus.Init,
        readOnly: false,
        formType: FormType.Insert,
        currentTab: 0,
        loaded: false,
        updated: false
      },
      optional
    );
    return initialState;
  }

  handleCreateObject<S extends BaseFormState<T>>(state: S): S {
    const item = this.fnCreateObject();
    const status = PageStatus.Init;
    const toolbar = this.initToolbar();
    const formType = FormType.Insert;
    return Object.assign({}, state, {
      item,
      status,
      toolbar,
      formType
    });
  }

  handleLoadObject<S extends BaseFormState<T>>(state: S): S {
    const status = PageStatus.Searching;
    const loaded = false;
    return Object.assign({}, state, { status, loaded });
  }

  handleLoadObjectSuccess<S extends BaseFormState<T>>(payload: T, state: S): S {
    const item = payload;
    const status = PageStatus.Init;
    const loaded = true;
    let toolbar;
    let formType;
    toolbar = this.fnGetToolbarObjectLoaded(item);
    formType = FormType.Update;
    return Object.assign({}, state, {
      item,
      status,
      toolbar,
      formType,
      loaded
    });
  }

  handleSaveObject<S extends BaseFormState<T>>(state: S): S {
    const updated = false;
    return Object.assign({}, state, { updated });
  }

  handleSaveObjectSuccess<S extends BaseFormState<T>>(payload: T, state: S): S {
    const item = payload;
    let toolbar = this.fnGetToolbarObjectLoaded(item);
    const formType = FormType.Update;
    const status = PageStatus.FinishSearch;
    const updated = true;
    return Object.assign({}, state, { item, formType, status, updated, toolbar });
  }

  handleSaveObjectFail<S extends BaseFormState<T>>(state: S): S {
    const status = PageStatus.Error;
    const updated = false;
    return Object.assign({}, state, { status, updated });
  }

  handleSetFormReadOnly<S extends BaseFormState<T>>(payload: boolean, state: S): S {
    const readOnly = payload;
    return Object.assign({}, state, { readOnly });
  }

  getSelectors(): BaseFormSelectors<T> {
    return {
      getItem: (state: BaseFormState<T>) => state.item,
      getToolbar: (state: BaseFormState<T>) => (state.readOnly ? {} : state.toolbar),
      getStatus: (state: BaseFormState<T>) => state.status,
      getFormType: (state: BaseFormState<T>) => state.formType,
      getCurrentTab: (state: BaseFormState<T>) => state.currentTab,
      getReadOnly: (state: BaseFormState<T>) => state.readOnly,
      getLoaded: (state: BaseFormState<T>) => state.loaded,
      getUpdated: (state: BaseFormState<T>) => state.updated
    };
  }

  initToolbar() {
    return {
      //mínimos toolbar de un formulario
      save: { visible: true, enable: true },
      delete: { visible: false, enable: true },
      print: { visible: false, enable: false }
    };
  }
}

export function createBaseFormAdapter<T>(
  fnCreateObject: FactoryCreateNewObject<T>,
  fnGetToolbarObjectLoaded?: FactoryCrateToolbar<T>
): BaseFormAdapter<T> {
  return new BaseFormAdapter(
    fnCreateObject,
    fnGetToolbarObjectLoaded ? fnGetToolbarObjectLoaded : getToolbarObjectLoadedDefault
  );
}

export function getToolbarObjectLoadedDefault() {
  return {
    save: { visible: true, enable: true },
    delete: { visible: true, enable: true },
    print: { visible: true, enable: true }
  };
}

export declare type BaseFormSelectors<T> = {
  getItem: (state: BaseFormState<T>) => T;
  getToolbar: (state: BaseFormState<T>) => any;
  getStatus: (state: BaseFormState<T>) => PageStatus;
  getFormType: (state: BaseFormState<T>) => FormType;
  getCurrentTab: (state: BaseFormState<T>) => number;
  getReadOnly: (state: BaseFormState<T>) => boolean;
  getLoaded: (state: BaseFormState<T>) => boolean;
  getUpdated: (state: BaseFormState<T>) => boolean;
};
