import { EntityState, EntityAdapter, createEntityAdapter, Dictionary } from "@ngrx/entity";

import { FormType } from "../../shared/global.constants";

import { FactoryCreateNewObject } from "./types";

export interface BaseSubListState<T> extends EntityState<T> {
  toolbar: any;
  modeInnerForm: FormType; //modo de edición del formulario: Insert, Update
  itemSelected: T; //elemento que debe mostrarse en el formulario interno
  displayInnerForm: boolean; //cuando debe verse el formulario interno del componente
  itemsSelected: number[]; //elementos seleccionados del listado mediante sus checks respectivos
}

export class BaseSubListAdapter<T> {
  private adapter: EntityAdapter<T>;

  constructor(args?: any) {
    this.adapter = createEntityAdapter<T>(args);
  }

  getInitialState<S extends BaseSubListState<T>>(makeItem: FactoryCreateNewObject<T>, optional?: any): S {
    const initialState = Object.assign(
      {
        toolbar: makeToolbarSubList(false),
        modeInnerForm: FormType.Insert,
        itemSelected: makeItem(),
        displayInnerForm: false,
        itemsSelected: []
      },
      optional
    );
    return this.adapter.getInitialState(initialState);
  }

  handleLoadSucces<S extends BaseSubListState<T>>(payload: T[], state: S): S {
    return this.adapter.addAll(payload, state);
  }

  handleCloseForm<S extends BaseSubListState<T>>(state: S): S {
    const displayInnerForm = false;
    return Object.assign({}, state, {
      displayInnerForm
    });
  }

  handleShowForm<S extends BaseSubListState<T>>(payload: T, formType: FormType, state: S): S {
    const displayInnerForm = true;
    const modeInnerForm = formType;
    const itemSelected = payload;
    return Object.assign({}, state, {
      displayInnerForm,
      modeInnerForm,
      itemSelected
    });
  }

  handleItemsSelected<S extends BaseSubListState<T>>(payload: number[], state: S): S {
    let toolbar;
    if (payload && payload.length > 0) {
      toolbar = makeToolbarSubList(true);
    } else {
      toolbar = makeToolbarSubList(false);
    }
    const itemsSelected = payload;
    return Object.assign({}, state, {
      toolbar,
      itemsSelected
    });
  }

  handleSaveItem<S extends BaseSubListState<T>>(payload: T, state: S): S {
    const displayInnerForm = false;
    const newState = this.adapter.upsertOne(payload, state);
    return Object.assign({}, newState, {
      displayInnerForm
    });
  }

  handleDeleteItems<S extends BaseSubListState<T>>(payload: number[], state: S): S {
    const displayInnerForm = false;
    const toolbar = makeToolbarSubList(false);
    const itemsSelected = [];
    const newState = this.adapter.removeMany(payload, state);
    return Object.assign({}, newState, {
      displayInnerForm,
      toolbar,
      itemsSelected
    });
  }

  getEntityAdapter() {
    return this.adapter;
  }

  getEntityAdapterSelectors() {
    return this.adapter.getSelectors();
  }

  getSelectors(): BaseSubListSelectors<T> {
    return {
      getToolbar: (state: BaseSubListState<T>) => state.toolbar,
      getModelInnerForm: (state: BaseSubListState<T>) => state.modeInnerForm,
      getItemSelected: (state: BaseSubListState<T>) => state.itemSelected,
      getDisplayInnerForm: (state: BaseSubListState<T>) => state.displayInnerForm,
      getItemsSelected: (state: BaseSubListState<T>) => state.itemsSelected
    };
  }
}

export function createBaseSubListAdapter<T>(): BaseSubListAdapter<T> {
  return new BaseSubListAdapter();
}

export function makeToolbarSubList(enableDelete: boolean) {
  return {
    new: { enable: true, visible: true },
    delete: { enable: enableDelete, visible: enableDelete }
  };
}

export declare type BaseSubListSelectors<T> = {
  getToolbar: (state: BaseSubListState<T>) => any;
  getModelInnerForm: (state: BaseSubListState<T>) => FormType;
  getItemSelected: (state: BaseSubListState<T>) => T;
  getDisplayInnerForm: (state: BaseSubListState<T>) => boolean;
  getItemsSelected: (state: BaseSubListState<T>) => number[];
};
