import { Observable }     from 'rxjs'; 

import {BaseCRUDServiceInterface} from './base-interface.service';
import {BaseFilter}         from './base.filter';
import {Item}               from '../../shared/data/item';
import {FormType}           from '../../shared/global.constants';
import {HttpUtilsService}   from '../../shared/service/http-utils.service';
import {MessageService}     from '../../shared/service/message.service';

/**
 * Clase básica que implementa un CRUD.
 * Es recomendable que todos los servicios que implementan un CRUD extiendan esta clase
 * o implementen la interface BaseCRUDServiceInterface.
 */
export abstract class BaseCRUDService implements BaseCRUDServiceInterface
{
   /** URL base del servicio CRUD. */
   protected baseUrl: string;
   
   constructor(
     protected messageService: MessageService,
     protected http: HttpUtilsService
   ) {}         

   /** Inserta o modifica la información del elemento. */
   save<T>(object: Item, mode: FormType, params?:{[key:string]:string}): Observable<T> {
      if (mode == FormType.Insert){
         let resolveUrl = this.resolveUrl(this.baseUrl, params);
         return this.http.getHttpPost<T>(resolveUrl, object);                  
      }
      else {
         let resolveUrl = this.resolveUrl(this.baseUrl + "/" + object.id, params);         
         return this.http.getHttpPut<T>(resolveUrl, object);
      }
   }

   /** Inserta o modifica la información del elemento. */
   saveObject<T>(object: Item, params?:{[key:string]:string}): Observable<T> {
      return this.save<T>(object, object && object.id ? FormType.Update: FormType.Insert, params)
   }
   
   /** Elimina el elemento. */
   delete(id: Number, params?:{[key:string]:string}) {
      let resolveUrl = this.resolveUrl(this.baseUrl + "/" + id, params);      
      return this.http.getHttpDelete(resolveUrl);
   }
   
   /** Elimina todos los elementos pasados por parámetro */
   deleteAll(items: number[], params?:{[key:string]:string}): Observable<any>{
      let resolveUrl = this.resolveUrl(this.baseUrl + "/delete", params);      
      return this.http.getHttpPost(resolveUrl, items);
   }

   /** Recupera la información del elemento. */
   get<T>(id: Number, params?:{[key:string]:string}): Observable<T> {
      let resolveUrl = this.resolveUrl(this.baseUrl + "/" + id, params);      
      return this.http.getHttpGet<T>(resolveUrl);
   }

   /** Recupera todos los elementos. */
   getAll<T>(params?:{[key:string]:string}): Observable<T> {
      let resolveUrl = this.resolveUrl(this.baseUrl, params);      
      return this.http.getHttpGet<T>(resolveUrl);
   }
   
   /** Recupera la lista (opcionalmente, filtrada y paginada) de elementos. */
   getAllByFilter<T>(filter: BaseFilter, params?:{[key:string]:string}): Observable<T> {
      let resolveUrl = this.resolveUrl(this.baseUrl + "/filter", params);      
      return this.http.getHttpPost<T>(resolveUrl, filter);
   }
   
   /** Recupera el número total de la lista (filtrada) de elementos. */
   countAllByFilter(filter: BaseFilter, params?:{[key:string]:string}): Observable<number> {
      let resolveUrl = this.resolveUrl(this.baseUrl + "/count", params);      
      return this.http.getHttpPost<number>(resolveUrl, filter);
   }

   /**
    * Construye la URL final en base a la plantilla y la colección de parámetros.
    * Si la clave de un parámetro forma parte de la URL, la reemplaza por su valor.
    * Si la clave de un parámetro no forma parte de la URL, se añadirá como parte del queryString.
    */
   resolveUrl(url:string, params:{[key:string]:string}): string{
      let queryParams: Array<string> = [];
      let newUrl = url;
      if (params){
          for (let key in params)
            if(newUrl.indexOf(key) >= 0) {
              newUrl = newUrl.replace(key, params[key]);
            } else {
              queryParams.push(`${key}=${params[key]}`);
            }   
      }
      if(queryParams.length > 0) {
        if(newUrl.indexOf('?') < 0) {
          newUrl += '?' + queryParams.join('&');
        } else {
          newUrl += '&' + queryParams.join('&');
        }
      }
      return newUrl;
   }
}