import {OnInit,
    Component, 
    forwardRef, 
    OnChanges,
    Input,
    Output,
    SimpleChange,
    EventEmitter}           from '@angular/core';
import {FormControl,
    ControlValueAccessor,
    NG_VALUE_ACCESSOR}      from '@angular/forms';
import {ActivatedRoute}         from '@angular/router';

import {AyudaService}           from '../service/ayuda.service';
import {SessionSharedService}   from '../service/session-shared.service';

const noop = () => {};

export const CUSTOM_SELECT_MULTIPLE_CONTROL_VALUE_ACCESSOR: any = {
provide: NG_VALUE_ACCESSOR,
useExisting: forwardRef(() => BitSelectMultipleComponent),
multi: true
};

/** 
* Clase que utiliza un custom componente para select múltiple de elementos
* Utiliza la interfaz ControlValueAccessor
*    <bit-select [domain]="'DOMAIN_TIPO_LOG'" id="tipolog" [(ngModel)]="filter.tipoLog" [ngModelOptions]="{standalone: true}">&nbsp;&nbsp;Tipus d'operació</bit-select>
* 
* @input data: array de objetos a mostrar en el select. Puede ser null, en cuyo caso domain no puede ser null
* @input domain: nombre del dominio que contiene los datos y que nos resolverá la página cuando cargue el componente. Puede ser null, en cuyo caso data no puede ser null
* @input id: identificador del componente select 
**/ 
@Component({
selector: 'bit-select-multiple',
template: `
            <div class="row separador" *ngIf="showSeparateSelectedLabels; else normalBitSelectMultiple">
                <div class="col-sm-3">
                    <ng-container *ngTemplateOutlet="normalBitSelectMultiple"></ng-container>
                </div>
                <div class="col-sm-9"><!-- elementos seleccionados -->
                    <label>&nbsp;</label>
                    <div class="multiselect-labels">
                        <div class="label label-primary multiselect-labels-lg" *ngFor="let item of value">
                        <span>{{labelSelectedItem(item)}}</span>
                        <a (click)="deleteSelectedItem($event, item , 'categoriaPrincipal')" title="Esborra categoria"> <i class="fa fa-times"></i></a>
                        </div>
                    </div>
                </div>
            </div>
            <ng-template #normalBitSelectMultiple>
            <label *ngIf="!hideLabel"><ng-content></ng-content>&nbsp;<span *ngIf="obligatorio" class="fa fa-asterisk"></span>
            &nbsp;&nbsp;
                <i *ngIf="ayuda" (click)="showAyuda()" class="btn-ayuda fa fa-question"></i>
            </label>
            <p *ngIf="lectura || readOnly" class="lectura">{{label}}</p>
            <div *ngIf="!readOnly">
                <p-multiSelect *ngIf="!lectura" [options]="listaOpciones" [defaultLabel]="emptyText" [(ngModel)]="value" id="{{id}}" 
                    (onFocus)="onFocus()" (onBlur)="onBlur()" (onChange)="changed(event)" [maxSelectedLabels]="maxSelectedLabels"
                    selectedItemsLabel="{0} elements seleccionats">
                </p-multiSelect>
            <control-messages *ngIf="control!=null" [control]="control" [field]="id"></control-messages>
            </div>
            </ng-template>
`,
providers: [CUSTOM_SELECT_MULTIPLE_CONTROL_VALUE_ACCESSOR]
})
export class BitSelectMultipleComponent implements ControlValueAccessor, OnInit, OnChanges {
@Input() control: FormControl;
@Input() id: any;
@Input() ayuda: string; // clave de la ayuda contextual
@Input() lectura: boolean = false;
@Input() readOnly: boolean = false;
@Input() emptyText: string = 'Cap';
@Input() hideLabel: boolean = false; // forzar no reservar espacio para el nombre del campo
@Input() maxSelectedLabels: number = 3;
@Input() showSeparateSelectedLabels: boolean = false; // indica si se muestran los elementos seleccionados en una columna separada
                                                        // Si se pueden seleccionar muchos elementos, que el label de los elementos seleccionados no 
                                                        // aparezca dentro del select, y hacerlo como una lista fuera del select.
listaOpciones: any[] = [];

@Input() private data: any[];
@Input() private domain: string;
@Input() private forceObligatorio: boolean = false;
@Output() onChange = new EventEmitter<string>();

private innerValue: any = '';
private onTouchedCallback: () => void = noop;
private onChangeCallback: (_: any) => void = noop;

constructor(
    private router: ActivatedRoute,
    private sessionShared: SessionSharedService,
    protected ayudaService: AyudaService
) {}

ngOnInit(){
    //console.log("Vamos a mapear los datos precargados");
    if (this.data == null && this.domain != null){
        //esta manera retornaría los datos alojados en el snapshot pero no tendríamos refresco en el caso de producirse
        //this.tiposLog = this.getTiposLogFromDomain(this.route.snapshot.data[TipoLog.DOMAIN_TIPO_LOG]);//this.route.data.pluck(TipoLog.DOMAIN_TIPO_LOG);
        //esta manera nos permite subscribirnos a los datos de data con lo que si hay refresco nos enteramos
        this.router.data.subscribe(
            result => {
                let dominios = result['dominios'];
                let domainTipo = dominios[this.domain];                                         
                if (domainTipo != null){
                    domainTipo.subscribe(
                        result => {                                    
                            // console.log(result);
                            this.sessionShared.set(this.domain, result.slice()); // almacenamos en sesión una copia para evitar transmitir cambios
                            this.listaOpciones = result;
                            this.postRetrieveData();
                        },
                        error => {this.listaOpciones = [{value: null, label: 'Error càrrega dominis'}];}
                    );                            
                }else{
                    console.log("No hay datos para el dominio",this.domain);
                    this.listaOpciones = [{value: null, label: 'Error càrrega dominis'}];
                }
            });
    } else if(this.data) {
        // incluso si la lista de valores ha sido proporcionada,
        // hay que añadir algunos comportamientos posteriores
        this.listaOpciones = this.data.slice();
        this.postRetrieveData();
    }
    //console.log("Componente inicializado");
}

ngOnChanges(changes: {[propKey: string]: SimpleChange}) {
    for (let propName in changes) {
        let changedProp = changes[propName];
        if (propName === 'data'){
            this.listaOpciones = changedProp.currentValue ? changedProp.currentValue.slice() : [];
            this.postRetrieveData();
        } else if(propName === 'dominio') {
            this.domain = changedProp.currentValue;
            // TODO: recargar dominio por si es distinto?
        }
    }
}

// custom processing after retrieving data array from service
postRetrieveData() {
}

//get accessor
get value(): any {
    return this.innerValue;
};

get label(): any {
    let val = this.value;
    let labels: any[] = this.listaOpciones && this.listaOpciones
                                                .filter(d => val && val.indexOf(d.value) > -1)
                                                .map(d => d.label);
    return labels ? labels.join(", ") : null;
}

//set accessor including call the onchange callback
set value(v: any) {
    if (v !== this.innerValue) {
        this.innerValue = v;
    }
    //this.onChangeCallback(v);
}

changed(event) {
    this.onTouchedCallback();
    this.onChangeCallback(this.innerValue);  //TODO: probar que esto es así
    this.onChange.emit(event);
}

onBlur() {
    this.onTouchedCallback();
}

onFocus() {      
    this.onTouchedCallback();
    // this.onChangeCallback(this.innerValue);
}

writeValue(value: any) {
    if (value !== this.innerValue) {
        this.innerValue = value;
    }
}

registerOnChange(fn: any) {
    this.onChangeCallback = fn;
}

registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
}

// decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
// o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
get obligatorio(): boolean {
    return this.forceObligatorio || (this.control && this.control.errors && this.control.errors['required']);
}

showAyuda() {
    this.ayudaService.mostrarAyuda(this.ayuda);
}

labelSelectedItem(id): string {
    return this.listaOpciones.find(c => c.value == id).label
}

deleteSelectedItem(event, item, tipoCategoria) {
    event.preventDefault();
    let index = this.value.indexOf(item);
    this.value.splice(index, 1);
    // se clona el array. Si no, el texto "{0} elementos seleccionados" del select de Prime no se actualiza.
    this.value = this.value.slice(0);
    this.changed(event);
}

}
