import {
    OnInit,
    Component,
    forwardRef,
    OnChanges,
    Input,
    Output,
    SimpleChange,
    EventEmitter
} from '@angular/core';
import {
    FormControl,
    ControlValueAccessor,
    NG_VALUE_ACCESSOR
} from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { AyudaService } from '../service/ayuda.service';
import { SessionSharedService } from '../service/session-shared.service';

const noop = () => { };

export const CUSTOM_SELECT_CONTROL_VALUE_ACCESSOR: any = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => BitSelectComponent),
    multi: true
};

/** 
* Clase que utiliza un custom componente para select de elementos
* Utiliza la interfaz ControlValueAccessor
*    <bit-select [domain]="'DOMAIN_TIPO_LOG'" id="tipolog" [(ngModel)]="filter.tipoLog" [ngModelOptions]="{standalone: true}">&nbsp;&nbsp;Tipus d'operació</bit-select>
* 
* @input data: array de objetos a mostrar en el select. Puede ser null, en cuyo caso domain no puede ser null
* @input domain: nombre del dominio que contiene los datos y que nos resolverá la página cuando cargue el componente. Puede ser null, en cuyo caso data no puede ser null
* @input id: identificador del componente select 
* @input filter: true indicará que podemos buscar dentro del select, false que no.
**/
@Component({
    selector: 'bit-select',
    template: `<label *ngIf="!hideLabel"><ng-content></ng-content>&nbsp;<span *ngIf="obligatorio" class="obligatorio fa fa-asterisk"></span>
            &nbsp;&nbsp;
                <i *ngIf="ayuda" (click)="showAyuda()" class="btn-ayuda fa fa-question"></i>
           </label>
           <p *ngIf="lectura || readOnly" class="lectura" [id]="nombre">{{label}}</p>
           <div *ngIf="!readOnly">
                <p-dropdown *ngIf="!lectura" [options]="listaOpcionesVisible" [(ngModel)]="value" [filter]="filter" id="{{nombre}}" [autoWidth]="false" (onChange)="changed($event)" (onFocus)="onFocus()" (onBlur)="onBlur()">
                        <ng-template let-item pTemplate="item">
                            <span [ngClass]="{'opcion-desactivada': item.inactivo}">{{item.label}}</span>
                        </ng-template>
                </p-dropdown>
                <control-messages *ngIf="control!=null" [control]="control" [field]="id"></control-messages>
           </div>
`,
    providers: [CUSTOM_SELECT_CONTROL_VALUE_ACCESSOR]
})
export class BitSelectComponent implements ControlValueAccessor, OnInit, OnChanges {
    @Input() control: FormControl;
    @Input() nombre: any;
    @Input() filter: boolean;
    @Input() ayuda: string; // clave de la ayuda contextual
    @Input() lectura: boolean = false;
    @Input() readOnly: boolean = false;
    @Input() excludedValues: any[];
    @Input() hideLabel: boolean = false; // forzar no reservar espacio para el nombre del campo
    listaOpciones: any[] = [];
    listaOpcionesVisible: any[] = [];
    @Input() data: any[];
    @Input() domain: string;
    @Input() emptyText: string = 'Qualsevol';
    @Input() includeEmpty: boolean = true;
    @Input() private forceObligatorio: boolean = false;

    @Output() onChange = new EventEmitter<string>();

    private innerValue: any = '';
    private onTouchedCallback: () => void = noop;
    private onChangeCallback: (_: any) => void = noop;

    constructor(
        private router: ActivatedRoute,
        private sessionShared: SessionSharedService,
        protected ayudaService: AyudaService
    ) { }

    ngOnInit() {
        //console.log("Vamos a mapear los datos precargados");
        if (this.filter === undefined) { this.filter = false; }
        if (this.data == null && this.domain != null) {
            //esta manera retornaría los datos alojados en el snapshot pero no tendríamos refresco en el caso de producirse
            //this.tiposLog = this.getTiposLogFromDomain(this.route.snapshot.data[TipoLog.DOMAIN_TIPO_LOG]);//this.route.data.pluck(TipoLog.DOMAIN_TIPO_LOG);
            //esta manera nos permite subscribirnos a los datos de data con lo que si hay refresco nos enteramos
            this.router.data.subscribe(
                result => {
                    let dominios = result['dominios'];
                    let domainTipo = dominios[this.domain];
                    if (domainTipo != null) {
                        domainTipo.subscribe(
                            result => {
                                //console.log(result);
                                this.sessionShared.set(this.domain, result.slice()); // almacenamos en sesión una copia para evitar transmitir cambios
                                this.listaOpciones = result.slice();
                                this.postRetrieveData();
                            },
                            error => { this.listaOpciones = [{ value: null, label: 'Error carrega dominis' }]; }
                        );
                    } else {
                        console.log("No hay datos para el dominio", this.domain);
                        this.listaOpciones = [{ value: null, label: 'Error carrega dominis' }];
                    }
                });
        } else if (this.data) {
            // incluso si la lista de valores ha sido proporcionada,
            // hay que añadir algunos comportamientos posteriores
            this.listaOpciones = this.data.slice();
            this.postRetrieveData();
        }
        //console.log("Componente inicializado");
    }

    ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
        let hayCambios: boolean = false;
        for (let propName in changes) {
            let changedProp = changes[propName];
            if (propName === 'data') {
                this.listaOpciones = changedProp.currentValue ? changedProp.currentValue.slice() : [];
                hayCambios = true;
            } else if (propName === 'dominio') {
                this.domain = changedProp.currentValue;
                // TODO: recargar dominio por si es distinto?
            } else if (propName === 'excludedValues') {
                this.excludedValues = changedProp.currentValue;
                hayCambios = true;
            } else if (propName === 'includeEmpty') {
                this.includeEmpty = changedProp.currentValue;
                hayCambios = true;
            }
        }
        if (hayCambios) { this.postRetrieveData(); }
    }

    // custom processing after retrieving data array from service
    postRetrieveData() {
        if (this.listaOpciones) {
            this.listaOpcionesVisible = Object.assign([], this.listaOpciones);
            if (this.includeEmpty) { // determinamos si debemos incluir la opción "sin selección" y con qué texto
                this.listaOpcionesVisible.splice(0, 0, { value: null, label: this.emptyText });
            }
            // repasamos si hay opciones a excluir que debamos eliminar
            if (this.excludedValues) {
                this.listaOpcionesVisible = this.listaOpcionesVisible.filter(op => this.excludedValues.indexOf(op.value) < 0);
            }
        }
    }

    //get accessor
    get value(): any {
        return this.innerValue;
    };

    get label(): any {
        let val = this.value;
        let dom = this.listaOpciones && this.listaOpciones.find(d => d.value == val);
        return dom ? dom.label : null;
    }

    //set accessor including call the onchange callback
    set value(v: any) {
        if (v !== this.innerValue) {
            this.innerValue = v;
        }
        //this.onChangeCallback(v);
    }

    changed(event) {
        this.onTouchedCallback();
        this.onChangeCallback(this.innerValue);  //TODO: probar que esto es así
        this.onChange.emit(event.value);
    }

    onBlur() {
        this.onTouchedCallback();
    }

    onFocus() {
        this.onTouchedCallback();
        // this.onChangeCallback(this.innerValue);
    }

    writeValue(value: any) {
        if (value !== this.innerValue) {
            this.innerValue = value;
        }
    }

    registerOnChange(fn: any) {
        this.onChangeCallback = fn;
    }

    registerOnTouched(fn: any) {
        this.onTouchedCallback = fn;
    }

    // decidimos marcar el campo como obligatorio si se ha indicado a la fuerza (por parámetro)
    // o si el control de formulario asociado está saltando por ser obligatorio y no haberse rellenado
    get obligatorio(): boolean {
        return this.forceObligatorio || (this.control && this.control.errors && this.control.errors['required']);
    }

    showAyuda() {
        this.ayudaService.mostrarAyuda(this.ayuda);
    }

}
