import { ChangeDetectorRef } from "@angular/core";
import { ControlValueAccessor } from "@angular/forms";

import { AyudaService } from "../../service/ayuda.service";
import { BitGeneralComponent } from "./bit-general.component";

const noop = () => {};

/** Clase customizada para componentes personalizados que requieren implementar la interface ControlValueAccessor.
 * Tipicamente para aquellos componentes personalizados que trabajan sobre otros componentes no estándars HTML como
 * pueden ser los componentes de prime como switch, autocomplete, calendar, select, etc.
 */
export abstract class BitCustomComponent extends BitGeneralComponent implements ControlValueAccessor {
  protected onTouchedCallback: () => void = noop;
  protected onChangeCallback: (_: any) => void = noop;

  constructor(protected ayudaService: AyudaService, protected changeDetectorRef: ChangeDetectorRef) {
    super(ayudaService);
  }

  //sobreescribimos el getter para coger el valor del FormControl
  get value(): any {
    return this.control.value;
  }

  //sobreescribimos el setter del value para poder transformarlo
  set value(v: any) {
    v = this.transformData(v);
    if (v !== this.control.value) {
      this.control.setValue(v);
      this.onTouchedCallback();
      this.onChangeCallback(v);
    }
  }

  /** Solo se activa este método si el input readOnly es true */
  get value_lectura(): string {
    if (this.value == null) {
      return "";
    } else {
      return this.value;
    }
  }

  /** El comportamiento normal es no transformar nada. Si el componente hijo necesita transformarlo
   * que sobreecriba el método y modifique el comportamiento. Esto se dará típicamente en los campos fecha porque
   * no queremos la versión ISO que nos da Prime si no una versión españolizada*/
  transformData(v) {
    return v;
  }

  /** Controlamos si debemos modificar el valor interno del componente mediante una transformacion al tener un pipe asociado*/
  onBlur() {
    if (this.instanciaPipe != null) {
      this.control.setValue(this.instanciaPipe.transform(this.control.value));
      this.onChangeCallback(this.control.value);
    }
    this.onTouchedCallback();
  }

  /** Controlamos si debemos modificar el valor interno del componente mediante una transformacion al tener un pipe asociado*/
  onFocus() {
    if (this.instanciaPipe != null) {
      this.control.setValue(this.instanciaPipe.parse(this.control.value));
      this.control.updateValueAndValidity();
      this.onChangeCallback(this.control.value);
    }
    this.onTouchedCallback();
  }

  /** Se activa cuando debemos Angular nos informa de que debemos actualizar la vista. Cada componente hijo deberá ver
   * si requiere o no implementar este método */
  writeValue(value: any) {
    if (value !== this.control.value) {
      // hack para que el ControlValueAccesor funcione correctamente cuando está referenciado en componentes con ChangeDetectionStrategy.OnPush
      // (no es nada bonito, pero Angular lleva 2 años con este bug y no lo han solucionado)
      // si no lo incluimos, en algunos casos se corre el riesgo de ir varios valores por detrás en el bit-input
      this.changeDetectorRef.markForCheck();
    }
  }

  registerOnChange(fn: any) {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any) {
    this.onTouchedCallback = fn;
  }
}
