import { Injectable } from "@angular/core";
import { FormGroup, FormControl, FormArray } from "@angular/forms";
import { GeneralUtils } from "./general-utils.service";

@Injectable()
export class CustomValidators {
  static config = {
    required: "Camp obligatori",
    invalidDate: "Data incorrecta",
    invalidEmailAddress: "Adreça electrònica incorrecta",
    invalidEmailCSVAddress: "Format adreça electrònica incorrecte",
    invalidLogin: "Login incorrecte",
    invalidEtiqueta: 'Etiqueta incorrecta, només s\'accepten valors alfanumèrics i "_"',
    invalidName: "Valor incorrecte",
    invalidCodigo: "Format de codi incorrecte",
    invalidAlfanumerico: "Format alfanumèric incorrecte",
    invalidCantidad: "Format incorrecte, només s'accepten valors numèrics (sense decimals)",
    invalidEnteroPositivo: "Introdueixi un valor numèric positiu (sense decimals) ",
    invalidEnteroPositivoIncluyeCero: "Introdueixi numèrics (sense decimals) majors o iguals que zero",
    invalidCantidadDecimal: "Format incorrecte, només s'accepten valors numèrics (amb decimals)",
    invalidNumeroPositivo: "Introdueixi un valor numèric positiu (amb decimals)",
    invalidCodigoPostal: "Introdueixi 5 dígits",
    invalidTelefono: "Introdueixi 9 dígits numèrics",
    invalidPorcentaje: "Percentatge incorrecte, només s'accepten valors numèrics entre 0 i 100",
    invalidUrl: "Adreça web incorrecta",
    invalidPath: "Ruta incorrecte",
    invalidNIF: "NIF incorrecte",
    invalidNIE: "NIE incorrecte",
    invalidCIF: "CIF incorrecte",
    invalidNIFNIE: "NIF/NIE incorrecte",
    invalidAnyo: "Format incorrecte, només s'accepten anys amb quatre dígits",
    maxlength: "Camp massa llarg",
    invalidNumeroDocumentoTipoDocumento: "Format del número de document incorrecte",
    direccionIncompleta: "El municipi i codi postal de l'adreça obligatoris",
    requiredExtraData: "Altres dades són obligatòries",
    invalidRangoFechaFinPosteriorInicio: "La data de fi ha de ser posterior a la data d'inici",
    invalidRangoHoraFinPosteriorInicio: "L'hora de fi ha de ser posterior a l'hora d'inici",
    invalidDateTimeNotAfterCurrentDate: "La data i hora han de ser posteriors a la data actual",

    // Cathos
    requiredProtocoloId: "Heu d'especificar un protocol d'inspecció",
    requiredTipoDatoTipoCheck:
      "Heu d'especificar si el tipus de dada aplica a tramitacions, inspeccions i/o autoavaluacions",

    // Bitcita
    agendaInvalidRangoFechaFinPosteriorInicio: "La data de fi ha de ser posterior a la data d'inici",
    agendaInvalidRangoFechaCampanya: "Les dates d'inici i de fi han de ser dins el període de la campanya",
    agendaInvalidSeleccionServicios: "Heu de seleccionar almenys un servei",
    agendaInvalidPlazasServicioMultiple: "Heu d'indicar el nombre de places del servei múltiple",
    agendaInvalidSeleccionHorario:
      "És necessari omplir almenys una hora d`inici i una hora de fi de qualsevol dia per guardar l'agenda",

    // CitaIVR Back
    llamadasCamposObligatorios:
      "Per favor, indiqui almenys un dels següents camps de cerca: transacció, telèfon, data, sol·licitant",
    transaccionCamposObligatorios: "Per favor, indiqui una transacció o una data de cerca"
  };

  static CustomValidatorsMap = {
    codigoNumerico: CustomValidators.codigoValidator,
    codigoAlfanumerico: CustomValidators.alfanumericoValidator,
    nombre: CustomValidators.nombreValidator,
    numero: CustomValidators.cantidadDecimalValidator,
    entero: CustomValidators.cantidadValidator,
    numeroPositivo: CustomValidators.numeroPositivoValidator,
    enteroPositivo: CustomValidators.enteroPositivoValidator,
    nif: CustomValidators.nifValidator,
    nie: CustomValidators.nieValidator,
    cif: CustomValidators.cifValidator,
    nifnie: CustomValidators.nifNieValidator,
    cp: CustomValidators.codigoPostalValidator,
    telefono: CustomValidators.telefonoValidator,
    email: CustomValidators.emailValidator,
    fecha: CustomValidators.dataValidator,
    anyo: CustomValidators.anyoValidator
  };

  static getValidatorErrorMessage(code: string) {
    return this.config[code];
  }

  // método para unir validadores
  static compose(validators: Function[]): Function {
    return function(g: FormGroup) {
      // ejecutamos cada uno de los validadores y unimos sus resultados
      return validators.map(validator => validator(g)).reduce((acc, result = {}) => Object.assign(acc, result), {});
    };
  }

  static required(control): { [s: string]: boolean } {
    if (!control.value) {
      return { required: true };
    } else {
      return null;
    }
  }

  // este validador en realidad no está haciendo nada porque bit-input, para fechas no válidas, sustituye
  // control.value por un null y el validador pasa siempre
  static dataValidator(control): { [s: string]: boolean } {
    // formato dd/MM/yyyy
    if (!control.value || control.value.match(/^(0[1-9]|1\d|2\d|3[01])\/(0[1-9]|1[0-2])\/(19|20)\d{2}$/)) {
      return null;
    } else {
      return { invalidDate: true };
    }
  }

  static timeValidator(control): { [s: string]: boolean } {
    // formato hh:mm
    if (!control.value || (control.value && control.value.match(/^[0-9][0-9]:[0-9][0-9]$/))) {
      return null;
    } else {
      return { invalidTime: true };
    }
  }

  static emailValidator(control): { [s: string]: boolean } {
    // RFC 2822 compliant regex
    if (!control.value || control.value.match(/\S+@([-\w]+\.)+\w+/g)) {
      return null;
    } else {
      return { invalidEmailAddress: true };
    }
  }

  static emailCSVValidator(control): { [s: string]: boolean } {
    // RFC 2822 compliant regex
    if (
      !control.value ||
      control.value.match(/^(( )*[\w\.]+@([-\w]+\.)+\w+( )*)(,( )*[\w\.]+@([-\w]+\.)+\w+( )*)*$/g)
    ) {
      return null;
    } else {
      return { invalidEmailCSVAddress: true };
    }
  }

  static loginValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value.match(/^[a-z0-9_-]{3,15}/)) {
      return null;
    } else {
      return { invalidLogin: true };
    }
  }

  static etiquetaValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value.match(/^[a-zA-Z0-9_-]+/)) {
      return null;
    } else {
      return { invalidEtiqueta: true };
    }
  }

  static nombreValidator(control): { [s: string]: boolean } {
    if (
      !control.value ||
      /^([A-Za-z0-9_\.\,\'\"\(\)\;\:\-\/\@\?\%\?\¿\&\+\-\*\/\ \n\rºª'´’·ïü?`áéíóúÁÉÍÓÚàèìòùÀÈÌÒÙÄËÏÖÜäëïöüñÑçÇ¡!#])+$/.test(
        control.value
      )
    ) {
      return null;
    } else {
      return { invalidName: true };
    }
  }

  static codigoValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value.match(/^([0-9])+$/)) {
      return null;
    } else {
      return { invalidCodigo: true };
    }
  }

  static alfanumericoValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value.match(/^([A-Z0-9_\.\-ÑÇ])+$/)) {
      return null;
    } else {
      return { invalidAlfanumerico: true };
    }
  }

  static cantidadValidator(control): { [s: string]: boolean } {
    // value is either empty, or a number without decimals
    if (!control.value || control.value === "") {
      return null;
    }
    let num = Number(control.value);
    if (!isNaN(num) && num % 1 == 0) {
      return null;
    }
    return { invalidCantidad: true };
  }

  /* Valida numeros enteros (sin decimales) positivos (>=0) */
  static enteroPositivoValidator(control): { [s: string]: boolean } {
    // value is either empty, or a number without decimals
    if (!control.value || control.value === "") {
      return null;
    }
    let num = Number(control.value);
    if (!isNaN(num) && num % 1 == 0 && num >= 0) {
      return null;
    }
    return { invalidEnteroPositivo: true };
  }

  static enteroPositivoValidatorIncluyeCeroValidator(control): { [s: string]: boolean } {
    // value is either empty, or a positive number (including 0) without decimals
    if (!control.value || control.value === "") {
      return null;
    }
    let num = Number(control.value);
    if (!isNaN(num) && num % 1 == 0 && num >= 0) {
      return null;
    }
    return { invalidEnteroPositivoIncluyeCero: true };
  }

  /* Valida numeros enteros (con decimales) */
  static cantidadDecimalValidator(control): { [s: string]: boolean } {
    // value is either empty, or a number (including decimals)
    if (!control.value || control.value === "" || !isNaN(Number(control.value))) {
      return null;
    } else {
      return { invalidCantidadDecimal: true };
    }
  }

  /* Valida numeros enteros (con decimales) positivos (>=0) */
  static numeroPositivoValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value === "" || (!isNaN(Number(control.value)) && Number(control.value) >= 0)) {
      return null;
    } else {
      return { invalidNumeroPositivo: true };
    }
  }

  /* Valida codigo postal (numero de 5 digitos) */
  static codigoPostalValidator(control): { [s: string]: boolean } {
    if (!control.value || (control.value.match(/^([0-9])+$/) && control.value.length == 5)) {
      return null;
    } else {
      return { invalidCodigoPostal: true };
    }
  }

  /* Valida telefono (numero de 9 digitos) */
  static telefonoValidator(control): { [s: string]: boolean } {
    if (!control.value || control.value.match(/^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$/)) {
      return null;
    } else {
      return { invalidTelefono: true };
    }
  }

  static buildLongitudValidator(lon: number) {
    let errorKey = `invalidLongitud-${lon}`;
    this.config[errorKey] = `La dada no pot excedir els ${lon} caràcters.`;
    return control => {
      if (!control.value || control.value.length <= lon) {
        return null;
      } else {
        let ret = {};
        ret[errorKey] = true;
        return ret;
      }
    };
  }

  static buildCantidadMinValidator(min: number) {
    let errorKey = `invalidCantidadMin-${min}`;
    this.config[errorKey] = `Format incorrecte. Només s\'accepten valors numèrics majors o iguals que ${min}`;
    return control => {
      let num = Number(control.value);
      if (control.value === "") {
        return null;
      } else if (isNaN(num) || num >= min) {
        // si no es numerico o el valor >= min, pasa la validacion
        return null;
      } else {
        let ret = {};
        ret[errorKey] = true;
        return ret;
      }
    };
  }

  static buildCantidadMaxValidator(max: number) {
    let errorKey = `invalidCantidadMax-${max}`;
    this.config[errorKey] = `Format incorrecte. Només s\'accepten valors numèrics menors o iguals que ${max}`;
    return control => {
      let num = Number(control.value);
      if (control.value === "") {
        return null;
      } else if (isNaN(num) || num <= max) {
        // si no es numerico o el valor <= max, pasa la validacion
        return null;
      } else {
        let ret = {};
        ret[errorKey] = true;
        return ret;
      }
    };
  }

  static buildCantidadMinMaxValidator(min: number, max: number) {
    let errorKey = `invalidCantidadMinMax-${min}-${max}`;
    this.config[errorKey] = `Format incorrecte. Només s\'accepten valors numèrics entre ${min} i ${max}`;
    return control => {
      let num = Number(control.value);
      if (control.value === "") {
        return null;
      } else if (!isNaN(num) && num >= min && num <= max) {
        return null;
      } else {
        let ret = {};
        ret[errorKey] = true;
        return ret;
      }
    };
  }

  static porcentajeValidator(control): { [s: string]: boolean } {
    // expected a numeric value between 0 and 100
    if (!control.value || control.value === "") {
      return null;
    } else {
      let n = Number(control.value);
      if (!isNaN(n) && n >= 0 && n <= 100) {
        return null;
      } else {
        return { invalidPorcentaje: true };
      }
    }
  }

  static urlValidator(control): { [s: string]: boolean } {
    var pattern = new RegExp(
      "^(https?:\\/\\/)?" + // protocol
      "((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.?)+[a-z]{2,}|" + // domain name
      "((\\d{1,3}\\.){3}\\d{1,3}))" + // OR ip (v4) address
      "(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*" + // port and path
      "(\\?[;&a-z\\d%_.~+=-]*)?" + // query string
        "(\\#[-a-z\\d_]*)?$",
      "i"
    ); // fragment locator
    if (!control.value || pattern.test(control.value)) {
      return null;
    } else {
      return { invalidUrl: true };
    }
  }

  static pathValidator(control): { [s: string]: boolean } {
    var pattern = new RegExp(
      "^((\\/)?[-a-z\\d%_.~+]*)*" + // port and path
      "(\\?[;&a-z\\d%_.~+=-]*)?" + // query string
        "(\\#[-a-z\\d_]*)?$",
      "i"
    ); // fragment locator
    if (!control.value || pattern.test(control.value)) {
      return null;
    } else {
      return { invalidPath: true };
    }
  }

  /**
   * Retorna un string con todos los errores que contiene el formulario separados por saltos de línea.
   */
  static validateForm(form: FormGroup, prefix?: string): string[] {
    let mensajes: string[] = [];
    // errores a nivel de grupo
    for (let errorKey in form.errors) {
      let pr = prefix ? `${prefix}: ` : "";
      mensajes.push(`${pr}${CustomValidators.config[errorKey]}`);
    }

    for (let controlKey in form.controls) {
      let control = form.controls[controlKey];
      if (!control.valid) {
        // no es válido, hay mensajes que recuperar
        let controlLabel = prefix ? `${prefix} > ${controlKey}` : controlKey; // nombre apilado

        // detectamos si es un campo o un formulario anidado
        if (control instanceof FormGroup) {
          // es un formulario anidado, seguimos bajando recursivamente
          mensajes = mensajes.concat(CustomValidators.validateForm(control, controlLabel));
        } else if (control instanceof FormControl) {
          // es un campo final, añadimos sus mensajes de error
          for (let errorKey in control.errors) {
            mensajes.push(`${controlLabel}: ${CustomValidators.config[errorKey]}`);
          }
        } else if (control instanceof FormArray) {
          // es una lista de controles anidada, la recorremos
          control.controls &&
            control.controls.forEach((arrControl, index) => {
              let arrControlLabel = `${controlLabel} > ${index}`;
              if (arrControl instanceof FormGroup) {
                // es una lista de FormGroups, bajamos a ellos recursivamente
                mensajes = mensajes.concat(CustomValidators.validateForm(arrControl, arrControlLabel));
              } else if (arrControl instanceof FormControl) {
                // es una lista de FormControls, los tratamos desde aquí
                for (let errorKey in control.errors) {
                  mensajes.push(`${arrControlLabel}: ${CustomValidators.config[errorKey]}`);
                }
              } else {
                // ??? ¿Un array de arrays? No está previsto.
                console.log("[CustomValidators] Unexpected FormArray with nested FormArrays");
              }
            });
        }
      }
    }
    return mensajes;
  }

  /**
   * Recibe un objeto con las propiedades {fieldTipo, fieldNumero} y devuelve un validador de formulario para documentos de identificación.
   * El error que puede generarse es del tipo 'invalidNumeroDocumentoTipoDocumento'.
   */
  static buildNumeroDocumentoValidator({
    fieldTipo = "documentoTipo",
    fieldNumero = "documentoNumero"
  } = {}): Function {
    return function(g: FormGroup) {
      if (!g) {
        return null;
      }
      let documentoTipo = g.get(fieldTipo) && g.get(fieldTipo).value;
      let documentoNumero = g && g.get(fieldNumero) && g.get(fieldNumero).value;
      let invalid = false;
      if (documentoTipo && documentoNumero) {
        if (documentoTipo == "DNI") {
          invalid = !CustomValidators.isValidDNI(documentoNumero);
        } else if (documentoTipo == "CIF") {
          invalid = !CustomValidators.isValidCIF(documentoNumero);
        } else if (documentoTipo == "NIF") {
          //invalid = !documentoNumero.match(/^[0-9]{8}[a-z|A-Z]$/);
          invalid = !CustomValidators.isValidNIF(documentoNumero);
        } else if (documentoTipo == "NIE") {
          invalid = !CustomValidators.isValidNIE(documentoNumero);
        }
      }

      if (invalid) {
        return { invalidNumeroDocumentoTipoDocumento: true };
      } else {
        return null;
      }
    };
  }

  static nifValidator(control): { [s: string]: boolean } {
    if (!control.value || CustomValidators.isValidNIF(control.value)) {
      return null;
    } else {
      return { invalidNIF: true };
    }
  }

  static nieValidator(control): { [s: string]: boolean } {
    if (!control.value || CustomValidators.isValidNIE(control.value)) {
      return null;
    } else {
      return { invalidNIE: true };
    }
  }

  static nifNieValidator(control): { [s: string]: boolean } {
    if (!control.value || CustomValidators.isValidNIF(control.value) || CustomValidators.isValidNIE(control.value)) {
      return null;
    } else {
      return { invalidNIFNIE: true };
    }
  }

  static cifValidator(control): { [s: string]: boolean } {
    var pattern = new RegExp(/^[a-z|A-Z][0-9]{7}[a-z|A-Z|0-9]/); // TODO
    if (!control.value || pattern.test(control.value)) {
      return null;
    } else {
      return { invalidCIF: true };
    }
  }

  static anyoValidator(control): { [s: string]: boolean } {
    var pattern = new RegExp(/^[0-9]{4}$/); // TODO
    if (!control.value || pattern.test(control.value)) {
      return null;
    } else {
      return { invalidAnyo: true };
    }
  }

  static isValidDNI(value: string): boolean {
    return true; // ¿deberíamos validar algo? Con letra ya tenemos el NIF
  }

  // https://github.com/jquery-validation/jquery-validation/blob/master/src/additional/cifES.js
  static isValidCIF(value: string): boolean {
    var cifRegEx = new RegExp(/^([ABCDEFGHJKLMNPQRSUVW])(\d{7})([0-9A-J])$/gi);
    var letter = value.substring(0, 1), // [ T ]
      number = value.substring(1, 8), // [ P ][ P ][ N ][ N ][ N ][ N ][ N ]
      control = value.substring(8, 9), // [ C ]
      all_sum = 0,
      even_sum = 0,
      odd_sum = 0,
      i,
      n,
      control_digit,
      control_letter;

    function isOdd(n) {
      return n % 2 === 0;
    }

    // Quick format test
    if (value.length !== 9 || !cifRegEx.test(value)) {
      return false;
    }

    for (i = 0; i < number.length; i++) {
      n = parseInt(number[i], 10);
      // Odd positions
      if (isOdd(i)) {
        // Odd positions are multiplied first.
        n *= 2;
        // If the multiplication is bigger than 10 we need to adjust
        odd_sum += n < 10 ? n : n - 9;
        // Even positions
        // Just sum them
      } else {
        even_sum += n;
      }
    }

    all_sum = even_sum + odd_sum;
    control_digit = (10 - Number(all_sum.toString().substr(-1))).toString();
    control_digit = parseInt(control_digit, 10) > 9 ? "0" : control_digit;
    control_letter = "JABCDEFGHI".substr(control_digit, 1).toString();

    // Control must be a digit
    if (letter.match(/[ABEH]/)) {
      return control === control_digit;
      // Control must be a letter
    } else if (letter.match(/[KPQS]/)) {
      return control === control_letter;
    }
    // Can be either
    return control === control_digit || control === control_letter;
  }

  // https://github.com/jquery-validation/jquery-validation/blob/master/src/additional/nifES.js
  static isValidNIF(value: string): boolean {
    value = value.toUpperCase();
    // Basic format test
    if (!value.match("((^[A-Z]{1}[0-9]{7}[A-Z0-9]{1}$|^[T]{1}[A-Z0-9]{8}$)|^[0-9]{8}[A-Z]{1}$)")) {
      return false;
    }
    // Test NIF
    if (/^[0-9]{8}[A-Z]{1}$/.test(value)) {
      return "TRWAGMYFPDXBNJZSQVHLCKE".charAt(Number(value.substring(8, 0)) % 23) === value.charAt(8);
    }
    // Test specials NIF (starts with K, L or M)
    if (/^[KLM]{1}/.test(value)) {
      return value[8] !== "TRWAGMYFPDXBNJZSQVHLCKE".charAt(Number(value.substring(8, 1)) % 23);
    }
    return false;
  }

  static isValidNIE(value: string): boolean {
    // Un NIE sigue una validación muy similar a la de un DNI, con la diferencia
    // de que en lugar de ser ocho números + letra debe ser letra + siete números + letra,
    // siendo la primera letra X, Y o Z sustituible por 0, 1 o 2 respectivamente.
    var nieRegEx = new RegExp(/^[XYZ]{1}[0-9]{7}[TRWAGMYFPDXBNJZSQVHLCKET]{1}$/gi);

    // Si no tiene el formato básico, no es válido
    if (!nieRegEx.test(value)) {
      return false;
    }

    // Sustituimos la primera letra por el número equivalente y reaprovechamos la validación de NIF
    value = value
      .toString()
      .toUpperCase()
      .replace(/^[X]/, "0")
      .replace(/^[Y]/, "1")
      .replace(/^[Z]/, "2");

    return CustomValidators.isValidNIF(value);
  }

  static rangoFechasValidator({ fieldFechaInicio = "fechaInicio", fieldFechaFin = "fechaFin" } = {}): Function {
    return function(g: FormGroup) {
      if (!g) {
        return null;
      }
      let fechaInicio = g.get(fieldFechaInicio) && g.get(fieldFechaInicio).value;
      let fechaFin = g && g.get(fieldFechaFin) && g.get(fieldFechaFin).value;
      if (fechaInicio && fechaFin) {
        let inicio = GeneralUtils.stringShortToDate(fechaInicio);
        let fin = GeneralUtils.stringShortToDate(fechaFin);
        if (inicio > fin) {
          return { invalidRangoFechaFinPosteriorInicio: true, requiredTipoDatoTipoCheck: true };
        }
      }
      return null;
    };
  }

  static rangoHorasValidator({ fieldHoraInicio = "horaInicio", fieldHoraFin = "horaFin" } = {}): Function {
    return function(g: FormGroup) {
      if (!g) {
        return null;
      }
      let inicio =
        g.get(fieldHoraInicio) &&
        g.get(fieldHoraInicio).value &&
        GeneralUtils.stringToTime(g.get(fieldHoraInicio).value);
      let fin =
        g && g.get(fieldHoraFin) && g.get(fieldHoraFin).value && GeneralUtils.stringToTime(g.get(fieldHoraFin).value);
      if (inicio && fin && inicio >= fin) {
        return { invalidRangoHoraFinPosteriorInicio: true };
      }
      return null;
    };
  }

  static dateTimeNotAfterCurrentDateValidator({ fieldfecha = "fecha", fieldHora = "hora" } = {}): Function {
    return function(g: FormGroup) {
      if (!g) {
        return null;
      }
      let campoFecha = g.get(fieldfecha);
      let campoHora = g.get(fieldHora);
      if (
        campoFecha &&
        campoFecha.value &&
        campoFecha.value.match(/^(0[1-9]|1\d|2\d|3[01])\/(0[1-9]|1[0-2])\/(19|20)\d{2}$/)
      ) {
        let date = GeneralUtils.getStringAsDate(campoFecha.value);
        if (campoHora && campoHora.value && campoHora.value.match(/^[0-9][0-9]:[0-9][0-9]$/)) {
          date.setHours(campoHora.value.split(":")[0]);
          date.setMinutes(campoHora.value.split(":")[1]);
        }
        if (date < new Date()) {
          return { invalidDateTimeNotAfterCurrentDate: true };
        }
      }
      return null;
    };
  }

  static buildMinLongitudValidator(lon: number) {
    let errorKey = `invalidLongitud-${lon}`;
    this.config[errorKey] = `El camp ha de tenir almenys ${lon} caràcters.`;
    return control => {
      if (!control.value || control.value.length >= lon) {
        return null;
      } else {
        let ret = {};
        ret[errorKey] = true;
        return ret;
      }
    };
  }
}
