import {Injectable}             from '@angular/core';
import {Observable, ReplaySubject}   from 'rxjs';
import { map } from 'rxjs/operators';

import {BaseCRUDService}        from '../../base/shared/base.service';
import {HttpUtilsService}       from '../service/http-utils.service';
import {MessageService}         from '../service/message.service';
import {DomainService}          from '../service/domain.service';
import {SessionSharedService}   from '../service/session-shared.service';

/**
 Servicio que devuelve los valores de un dominio ofrecidos por DomainService pero
 añadiendo un nivel de caché en sesión.

 De este modo, si N componentes quieren consumir el mismo servicio no provocaremos N llamadas REST.

 Esto lo hacemos solo con dominios ya que sabemos que no cambiarán durante el ciclo de vida de la aplicación. 
 */
@Injectable()
export class DomainCachedService extends BaseCRUDService {

   private cachedResolvers: Map<string, ReplaySubject<any>>; // observables de los dominios ya solicitados en algun momento

   constructor(
        protected messageService: MessageService,
        protected http: HttpUtilsService,
        protected domainService: DomainService,
        protected sessionShared: SessionSharedService
    ) {
      super(messageService, http);
      this.cachedResolvers = new Map<string, ReplaySubject<any>>();
   }

    getDominio(dominio: string){
        // comprobamos si ya se ha pedido antes
        let replay: ReplaySubject<any> = this.cachedResolvers.get(dominio);
        if(replay) {
            // ya tenemos preparado un ReplaySubject para ese dominio, lo devolvemos tal cual
            // si el valor todavía no ha llegado de servidor, cuando llegue se emitirá
            // si el valor ya había llegado de servidor, como es un replay=1 se emitirá inmediatamente
        } else {
            // es la primera vez que nos piden este dominio, así que hacemos lo siguiente
            // creamos un ReplaySubject que repetirá el último valor recibido cada vez que se subscriban a él
            // resolvemos el dominio contra servidor, y su resolución la emitimos contra el replaysubject
            replay = new ReplaySubject(1); // solo repetiremos el último valor
            this.domainService.getDominio(dominio).subscribe(
                r => { replay.next(r); },
                err => {
                    // si ha fallado, nos libramos del replay para que el siguiente que lo pida vuelva a intentarlo
                    replay.error(err);
                    this.cachedResolvers.delete(dominio);
                }
            );
            this.cachedResolvers.set(dominio, replay);
        }
        // lo que devolvemos debe ser una copia, ya que si un componente manipula el objeto
        // el resto de componentes que usen el mismo dominio no deben enterarse
        return replay.pipe(map(res => JSON.parse(JSON.stringify(res))));
    }
   
}