import { Injectable, Inject } from "@angular/core";
import { HttpClient, HttpHeaders, HttpResponse, HttpErrorResponse } from "@angular/common/http";
import { Observable, throwError } from "rxjs";
import { tap } from "rxjs/operators";

import { Error as CustomError } from "../data/error";
import { LocatorService } from "./locator.service";
import { MessageService } from "./message.service";
import { GeneralUtils } from "./general-utils.service";

@Injectable()
export class HttpUtilsService {
  // customizado por proyecto: {provide: 'CONTEXT_PATH', useValue: environment.CONTEXT}
  static CONTEXT_PATH = "bitcitafront";
  static LOGIN_AUTHENTICATION;

  constructor(
    public http: HttpClient,
    public messageService: MessageService,
    public generalUtils: GeneralUtils,
    @Inject("API_URL") private apiUrl: string,
    @Inject("CONTEXT_PATH") private contextPath: string
  ) {
    let injector = LocatorService.injector;
    HttpUtilsService.CONTEXT_PATH = contextPath;
    console.log("HttpUtilsSerive creado con API Url:" + this.apiUrl);
  }

  private handleError(error: HttpErrorResponse) {
    this.generalUtils.stopLoading();
    let customError = HttpUtilsService.getCustomError(error);
    this.messageService.sendError(customError);
    return throwError(customError);
  }

  public static getCustomError(error: HttpErrorResponse): CustomError {
    let lastUrlAcceded = error.url;
    let innerError;
    let customError;

    if (
      error.message &&
      (error.message.includes("Unexpected token < in JSON") || //chrome
      error.message.includes("JSON.parse: unexpected character at line 1 column 1") || //firefox
        error.message.includes("JSON.parse Error: Invalid character at position:1")) //explorer
    ) {
      console.log("Parece que hemos perdido la sesión");
      //customError = new CustomError(lastUrlAcceded, "S'ha detectat que s'ha perdut la sessió amb el servidor. És necessari que torni a autenticar-se en el sistema.\n Si no apareix la pàgina d'introducció d'usuari i contrasenya, per favor tanqui el navegador i torni a obrir l'aplicació.",
      //"S'ha detectat que s'ha perdut la sessió amb el servidor. És necessari que torni a autenticar-se en el sistema.\n Si no apareix la pàgina d'introducció d'usuari i contrasenya, per favor tanqui el navegador i torni a obrir l'aplicació.", undefined, true, null);
      HttpUtilsService.reloadPage();
      return;
    }

    try {
      innerError = JSON.parse(error.error);
    } catch (err) {
      innerError = error.error;
    }
    if (innerError && (innerError["notificacion"] || innerError["trazaError"])) {
      //el error es controlado del servidor
      customError = <CustomError>innerError;
    } else {
      //el error es un problema de comunicaciones o del propio cliente Angular
      if (error.status || error.status === 0) {
        //caso jetty o error controlado del servidor
        console.log("HTTP Error con status " + error.status);
        if (error.status >= 300 && error.status < 500) {
          customError = new CustomError(
            lastUrlAcceded,
            "Error d'accés o de connexió: No s'ha trobat la informació sol·licitada",
            "Error de acceso o de conexión: No se ha encontrado la información solicitada",
            undefined,
            true,
            null
          );
        } else {
          customError = new CustomError(
            lastUrlAcceded,
            "S'ha produït un error en el servidor; per favor, esperi uns segons i torni a realitzar la mateixa acció",
            "Se ha producido un error en el servidor; por favor, espere unos segundos y vuelva a realizar la misma acción",
            undefined,
            true,
            null
          );
        }
      } else {
        customError = new CustomError(lastUrlAcceded, error.message, error.message, error.error);
      }
    }
    return customError;
  }

  static reloadPage() {
    let currentUrl = window.location.href;
    console.log(`current url ${currentUrl}`);
    let index: number = currentUrl.lastIndexOf(HttpUtilsService.CONTEXT_PATH);
    let newUrl: string;
    if (index != -1) {
      newUrl = currentUrl.substring(0, index + HttpUtilsService.CONTEXT_PATH.length);
    } else {
      newUrl = currentUrl;
    }
    console.log(`new url ${newUrl}`);
    window.location.href = newUrl;
  }

  public getHttpPost<T>(url, hqlRequest, emptyResponse: boolean = false): Observable<T> {
    console.log("calling url server via POST:" + this.apiUrl + url);
    this.generalUtils.startLoading();
    let body = JSON.stringify(hqlRequest);
    let headers: HttpHeaders = new HttpHeaders().set("Content-Type", "application/json");
    headers = this.addAuthenticationData(headers);
    let vResponseType: any = emptyResponse ? "text" : "json";
    return this.http
      .post<T>(this.apiUrl + url, body, { headers, responseType: vResponseType })
      .pipe(tap(res => this.generalUtils.stopLoading(), error => this.handleError(error)));
  }

  public getHttpPut<T>(url, hqlRequest): Observable<T> {
    console.log("calling url server via PUT:" + this.apiUrl + url);
    this.generalUtils.startLoading();
    let body = JSON.stringify(hqlRequest);
    let headers: HttpHeaders = new HttpHeaders().set("Content-Type", "application/json");
    headers = this.addAuthenticationData(headers);
    return this.http
      .put<T>(this.apiUrl + url, body, { headers })
      .pipe(tap(res => this.generalUtils.stopLoading(), error => this.handleError(error)));
  }

  public getHttpDelete(url): Observable<any> {
    console.log("calling url server via DELETE:" + this.apiUrl + url);
    this.generalUtils.startLoading();
    let headers: HttpHeaders = new HttpHeaders().set("Content-Type", "application/json");
    headers = this.addAuthenticationData(headers);
    return this.http
      .delete(this.apiUrl + url, {
        headers,
        responseType: "text"
      })
      .pipe(tap(res => this.generalUtils.stopLoading(), error => this.handleError(error)));
  }

  public getHttpGet<T>(url): Observable<T> {
    console.log("calling url server via GET:" + this.apiUrl + url);
    this.generalUtils.startLoading();
    let headers: HttpHeaders = this.addAuthenticationData(new HttpHeaders());

    return this.http
      .get<T>(this.apiUrl + url, {
        headers: headers
      })
      .pipe(tap(res => this.generalUtils.stopLoading(), error => this.handleError(error)));
  }

  private addAuthenticationData(httpHeaders: HttpHeaders) {
    if (HttpUtilsService.LOGIN_AUTHENTICATION && HttpUtilsService.LOGIN_AUTHENTICATION != "") {
      httpHeaders = httpHeaders.append("Authorization", "Basic " + HttpUtilsService.LOGIN_AUTHENTICATION);
    }
    return httpHeaders;
  }
}

export var httpUtilsServiceInjectables: Array<any> = [
  {
    provide: HttpUtilsService,
    useClass: HttpUtilsService
  }
];
