import { Component, OnInit, OnDestroy } from "@angular/core";
import { FormBuilder, Validators } from "@angular/forms";
import { Router, ActivatedRoute } from "@angular/router";
import { ObjetoTest } from "../shared/objeto";
import { TestService } from "../shared/test.service";
import { Error } from "../../shared/data/error";
import { ConfigurationEditor } from "../../shared/data/configuration-editor";
import { BaseFormComponent } from "../../base/component/base-form.component";
import { Domain } from "../../shared/data/domain";
import { MessageService } from "../../shared/service/message.service";
import { ToolbarService } from "../../shared/service/toolbar.service";
import { CustomValidators } from "../../shared/service/custom-validators.service";
import { NewObjetoTest } from "../shared/newobjeto";

/**
 * Componente de ejemplo de edición de un formulario.
 */
@Component({
  selector: "newformulario-form",
  templateUrl: "./newformulario-form.component.html"
})
export class NewFormularioFormComponent extends BaseFormComponent implements OnInit, OnDestroy {
  object: NewObjetoTest;
  displayModal: boolean;
  nombres: string[];
  listaNombres: ["Thor", "Iron man", "Spiderman", "Thanos", "Dr Strange", "Capitán América", "Capitana Marvel"];
  readOnly: boolean = false;

  private subscribeParams: any; // nos permitirá recibir los parametros de entrada (URL) a este componente
  public configurationEditor: ConfigurationEditor = ConfigurationEditor.newCustomObject({
    fontFormat: true,
    fontSize: true,
    bullet: true,
    link: true
  });
  datosSelectMultiple: any = [
    new Domain("1", "User 01"),
    new Domain("2", "User 02"),
    new Domain("3", "User 03"),
    new Domain("4", "User 04"),
    new Domain("5", "User 05"),
    new Domain("6", "User 06"),
    new Domain("7", "User 07"),
    new Domain("8", "User 08"),
    new Domain("9", "User 09")
  ];

  constructor(
    protected router: Router,
    protected testService: TestService,
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected toolbarService: ToolbarService,
    private formBuilder: FormBuilder
  ) {
    super(testService, router, activatedRoute, messageService, toolbarService);
    this.object = NewObjetoTest.newObject();
    this.myForm = this.formBuilder.group({
      id: [this.object.id],
      nombre: [this.object.nombre, [Validators.required, CustomValidators.nombreValidator]],
      fecha: [this.object.fecha, [CustomValidators.dataValidator]],
      fechaHora: [this.object.fechaHora],
      hora: [this.object.hora],
      activo: [this.object.activo],
      siempreActivo: [{ value: this.object.siempreActivo, disabled: true }],
      inputswitch: [this.object.inputSwitch],
      descripcion: [this.object.descripcion],
      autocomplete: [this.object.autocomplete],
      readonly: [false],
      usuario: [this.object.usuario, [Validators.required]],
      usuarios: [this.object.usuarios, [Validators.required]],
      empleados: [this.object.empleados, [Validators.required]],
      edicion: [this.object.edicion]
    });
  }

  /** Inicializa la ficha: lee el elemento con el id indicado e inicializa el componente. */
  ngOnInit() {
    /* Modo de obtener el parámetro via subscripción. Muy útil cuando estamos invocando al componente más de una vez sin recrearlo.
        Por ejemplo con botones de adelante o atrás. En ese caso no queremos que ngOnInit se invoque cada vez si no que se reutilice
        el componente y necesitamos detectar cuando el parámetro cambia.*/
    let id;
    this.subscribeParams = this.activatedRoute.params.subscribe(params => {
      let id = +params["id"]; // (+) converts string 'id' to a number
      this.init(id);
    });
  }

  /** Cierra la ficha: libera memoria, desvincula subscripciones, etc. */
  ngOnDestroy() {
    if (this.subscribeParams) {
      this.subscribeParams.unsubscribe(); // necesario para evitar memory leaks
    }
  }

  init(id) {
    super.init(id);
    //definimos la miga de pan en el OnInit porque necesitamos saber si estamos editando o creando una nueva normativa
    this.breadcrumb = [];
    //this.breadcrumb.push({ label: 'Manteniment'});
    if (!id) {
      this.breadcrumb.push({ label: "Objectes", url: "objeto" });
      this.breadcrumb.push({ label: "Nou objecte" });
    } else {
      this.breadcrumb.push({ label: "Objectes", url: "objeto", params: { query: "true" } });
      this.breadcrumb.push({ label: "Edició de objecte" });
    }
  }

  /** Limpia el formulario. */
  cleanForm(): void {
    // super.cleanForm();
    // this.object = PuntoInteres.newObject();
  }

  showModal() {
    console.log("show modal");
    this.displayModal = true;
  }

  onCloseModal() {
    this.displayModal = false;
  }

  /** Cierra el formulario y vuelve a la pantalla anterior. */
  goBack(): void {
    this.router.navigate(["puntoInteres", { query: "true" }]);
  }

  /** Información de debug. */
  get diagnostic() {
    return JSON.stringify(this.object);
  }

  save(): void {
    if (!this.myForm.valid) {
      let errores = CustomValidators.validateForm(this.myForm);
      this.messageService.sendError(
        new Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores)
      );
    } else {
      if (this.preSave()) {
        alert("saved:" + JSON.stringify(this.object));
      }
    }
  }

  get formReadOnly(): boolean {
    return this.myForm.get("readonly").value;
  }

  clickCheckbox(data) {
    console.log(`checkbox clicado: ${data}`);
  }

  searchNombre(event) {
    let query = event.query;
    this.nombres = [
      "Thor",
      "Iron man",
      "Spiderman",
      "Thanos",
      "Dr Strange",
      "Capitán América",
      "Capitana Marvel"
    ].filter(nombre => nombre.includes(query));
  }

  selectNombre(event) {
    alert(`${event} seleccionado`);
  }

  selectUsuario(event) {
    alert(`${event} seleccionado`);
  }

  selectUsuarios(event) {
    alert(`${event} seleccionado`);
  }
}
